@@ -2,7 +2,7 @@
 #
 # The PyMca X-Ray Fluorescence Toolkit
 #
-# Copyright (c) 2004-2018 European Synchrotron Radiation Facility
+# Copyright (c) 2004-2019 European Synchrotron Radiation Facility
 #
 # This file is part of the PyMca X-ray Fluorescence Toolkit developed at
 # the ESRF by the Software group.
@@ -33,10 +33,17 @@ __copyright__ = "European Synchrotron Radiation Facility, Grenoble, France"
 import sys
 import os
 import numpy
+try:
+    # try to import hdf5plugin
+    import hdf5plugin
+except:
+    # but do not crash just because of it
+    pass
 import h5py
 from operator import itemgetter
 import re
 import posixpath
+import logging
 phynx = h5py
 
 if sys.version_info >= (3,):
@@ -46,7 +53,38 @@ from . import DataObject
 from . import NexusTools
 
 SOURCE_TYPE = "HDF5"
-DEBUG = 0
+
+try:
+    from silx.io import open as silxh5open
+    logging.getLogger("silx.io.fabioh5").setLevel(logging.CRITICAL)
+except:
+    silxh5open = None
+
+def h5open(filename):
+    try:
+        # try to open as usual using h5py
+        return h5py.File(filename, "r")
+    except:
+        try:
+            if h5py.version.hdf5_version_tuple < (1, 10):
+                # no reason to try SWMR mode
+                raise
+            elif h5py.is_hdf5(filename):
+                _logger.info("Cannot open %s. Trying in SWMR mode" % filename)
+                return h5py.File(filename, "r", libver='latest', swmr=True)
+            else:
+                raise
+        except:
+            if silxh5open:
+                try:
+                    _logger.info("Trying to open %s using silx" % filename)
+                    return silxh5open(filename)
+                except:
+                    _logger.info("Cannot open %s using silx" % filename)
+            # give back the original error
+            return h5py.File(filename, "r")
+
+_logger = logging.getLogger(__name__)
 
 #sorting method
 def h5py_sorting(object_list):
@@ -75,7 +113,7 @@ def h5py_sorting(object_list):
 
     try:
         if sorting_key != 'name':
-            sorting_list = [(o[sorting_key].value, o)
+            sorting_list = [(o[sorting_key][()], o)
                            for o in object_list]
             sorted_list = sorted(sorting_list, key=itemgetter(0))
             return [x[1] for x in sorted_list]
@@ -89,8 +127,8 @@ def h5py_sorting(object_list):
         #The only way to reach this point is to have different
         #structures among the different entries. In that case
         #defaults to the unfiltered case
-        print("WARNING: Default ordering")
-        print("Probably all entries do not have the key %s" % sorting_key)
+        _logger.warning("Default ordering")
+        _logger.warning("Probably all entries do not have the key %s", sorting_key)
         return object_list
 
 def _get_number_list(txt):
@@ -141,29 +179,6 @@ def get_family_pattern(filelist):
         rootname = name1[0:]
     return rootname
 
-# tool to retrieve the positioners group associated to a path
-# retrieving them from the same entry (assuming they are in
-# NXentry/instrument/positioners)
-def getPositionersGroup(h5file, path):
-    entry = path
-    candidate = os.path.dirname(entry)
-    while len(candidate) > 1:
-        entry = candidate
-        candidate = os.path.dirname(entry)
-    instrument = None
-    for key, group in h5file[entry].items():
-        for attr in group.attrs:
-            if attr in ["NX_class", b"NX_class"]:
-                if group.attrs[attr] in ["NXinstrument", b"NXinstrument"]:
-                    instrument = group
-                    break
-    positioners = None
-    if instrument is not None:
-        for key in instrument.keys():
-            if key in ["positioners", b"positioners"]:
-                positioners = instrument[key]
-    return positioners
-
 class NexusDataSource(object):
     def __init__(self,nameInput):
         if type(nameInput) == type([]):
@@ -173,7 +188,7 @@ class NexusDataSource(object):
         self.sourceName = []
         for name in nameList:
             if not isinstance(name, basestring):
-                if not isinstance(name, phynx.File):
+                if not isinstance(name, h5py.File):
                     text = "Constructor needs string as first argument"
                     raise TypeError(text)
                 else:
@@ -191,17 +206,17 @@ class NexusDataSource(object):
         self._sourceObjectList=[]
         FAMILY = False
         for name in self.__sourceNameList:
-            if isinstance(name, phynx.File):
+            if isinstance(name, h5py.File):
                 self._sourceObjectList.append(name)
                 continue
             if not os.path.exists(name):
                 if '%' in name:
-                   phynxInstance = phynx.File(name, 'r',
+                   phynxInstance = h5py.File(name, 'r',
                                               driver='family')
                 else:
                     raise IOError("File %s does not exists" % name)
             try:
-                phynxInstance = phynx.File(name, 'r')
+                phynxInstance = h5open(name)
             except IOError:
                 if 'FAMILY DRIVER' in sys.exc_info()[1].args[0].upper():
                     FAMILY = True
@@ -209,7 +224,7 @@ class NexusDataSource(object):
                     raise
             except TypeError:
                 try:
-                    phynxInstance = phynx.File(name, 'r')
+                    phynxInstance = h5open(name)
                 except IOError:
                     if 'FAMILY DRIVER' in sys.exc_info()[1].args[0].upper():
                         FAMILY = True
@@ -225,7 +240,7 @@ class NexusDataSource(object):
         if FAMILY:
             pattern = get_family_pattern(self.__sourceNameList)
             if '%' in pattern:
-                phynxInstance = phynx.File(pattern, 'r',
+                phynxInstance = h5py.File(pattern, 'r',
                                             driver='family')
             else:
                 raise IOError("Cannot read set of HDF5 files")
@@ -262,8 +277,7 @@ class NexusDataSource(object):
             return self.__getKeyInfo(key)
         else:
             #should we raise a KeyError?
-            if DEBUG:
-                print("Error key not in list ")
+            _logger.debug("Error key not in list ")
             return {}
 
     def __getKeyInfo(self,key):
@@ -273,8 +287,7 @@ class NexusDataSource(object):
             entry = int(entry)-1
         except:
             #should we rise an error?
-            if DEBUG:
-                print("Error trying to interpret key = %s" % key)
+            _logger.debug("Error trying to interpret key = %s", key)
             return {}
 
         sourceObject = self._sourceObjectList[index]
@@ -291,10 +304,9 @@ class NexusDataSource(object):
         """
         key:  a string of the form %d.%d indicating the file and the entry
               starting by 1.
-        selection: a dictionnary generated via QNexusWidget
+        selection: a dictionary generated via QNexusWidget
         """
-        if DEBUG:
-            print("getDataObject selection = ", selection)
+        _logger.debug("getDataObject selection = %s", selection)
         if selection is not None:
             if 'sourcename' in selection:
                 filename  = selection['sourcename']
@@ -314,7 +326,7 @@ class NexusDataSource(object):
             actual_key = "%d.%d" % (fileIndex+1, entryIndex+1)
             if actual_key != key:
                 if entry != "/":
-                    print("Warning selection keys do not match")
+                    _logger.warning("selection keys do not match")
         else:
             #Probably I should find the acual entry following h5py_ordering output
             #and search for an NXdata plot.
@@ -332,6 +344,12 @@ class NexusDataSource(object):
         output.m = None
         output.data = None
         output.info = self.__getKeyInfo(actual_key)
+        try:
+            output.info["title"] = NexusTools.getTitle(phynxFile, entry)
+        except:
+            txt = "Error reading title for path <%s>"
+            _logger.warning(txt)
+            output.info["title"] = ""
         output.info['selection'] = selection
         if "mca" in selection:
             # this should go somewhere else
@@ -347,17 +365,16 @@ class NexusDataSource(object):
                     mcaDatasetObjectPath = mcaObjectPaths[key]
                     dataset = None
                     if mcaDatasetObjectPath in h5File:
-                        dataset = h5File[mcaDatasetObjectPath].value
+                        dataset = h5File[mcaDatasetObjectPath][()]
                     elif "::" in mcaDatasetObjectPath:
                         fileName, path = mcaDatasetObjectPath.split()
                         if os.path.exists(fileName):
-                            with h5py.File(fileName, "r") as h5:
+                            with h5open(fileName) as h5:
                                 if path in h5:
-                                    dataset = h5[path].value
+                                    dataset = h5[path][()]
                     if dataset is None:
-                        if DEBUG:
-                            print("Broken link? Ignoring key %s = % s" % \
-                                                  (key, mcaDatasetObjectPath ))
+                        _logger.debug("Broken link? Ignoring key %s = %s",
+                                      key, mcaDatasetObjectPath)
                         del mcaObjectPaths[key]
                     else:
                         mcaObjectPaths[key] = dataset
@@ -393,7 +410,7 @@ class NexusDataSource(object):
                         if selection['mcaselectiontype'].lower() != "sum":
                             mcaData /= divider
                     else:
-                        mcaData = mcaData.value
+                        mcaData = mcaData[()]
                         divider = 1.0
                     if "McaLiveTime" in output.info:
                         if numpy.isscalar(output.info["McaLiveTime"]):
@@ -403,11 +420,11 @@ class NexusDataSource(object):
                             output.info["McaLiveTime"] = \
                                     output.info["McaLiveTime"].sum()
                         if selection['mcaselectiontype'].lower() != "sum":
-                            output.info["McaLiveTime"] /= divider                        
+                            output.info["McaLiveTime"] /= divider
             except:
-                import traceback                
-                print("exception", sys.exc_info())
-                print(("%s " % value) + ''.join(traceback.format_tb(trace)))
+                # import traceback
+                _logger.error("%s", sys.exc_info())
+                # print(("%s " % value) + ''.join(traceback.format_tb(trace)))
                 return output
             output.x = [mcaChannels]
             output.y = [mcaData]
@@ -430,13 +447,17 @@ class NexusDataSource(object):
             output.info['LabelNames'] = selection['cntlist']
         if entry != "/":
             try:
-                positioners = getPositionersGroup(phynxFile, entry)
+                positioners = NexusTools.getPositionersGroup(phynxFile, entry)
                 if positioners is not None:
                     output.info['MotorNames'] = []
                     output.info['MotorValues'] = []
                     for key in positioners.keys():
+                        if positioners[key].dtype == numpy.object:
+                            # not a standard value
+                            _logger.info("Skipping object key %s" % key)
+                            continue
                         output.info['MotorNames'].append(key)
-                        value = positioners[key].value
+                        value = positioners[key][()]
                         if hasattr(value, "size"):
                             if value.size > 1:
                                 if hasattr(value, "flat"):
@@ -444,8 +465,7 @@ class NexusDataSource(object):
                         output.info['MotorValues'].append(value)
             except:
                 # I cannot affort to fail here for something probably not used
-                if DEBUG:
-                    print("Error reading positioners ", sys.exc_info())
+                _logger.debug("Error reading positioners\n%s", sys.exc_info())
         for cnt in ['y', 'x', 'm']:
             if not cnt in selection:
                 continue
@@ -458,7 +478,7 @@ class NexusDataSource(object):
                 totalElements *= dim
             if totalElements < 2.0E7:
                 try:
-                    data = phynxFile[path].value
+                    data = phynxFile[path][()]
                 except MemoryError:
                     data = phynxFile[path]
                     pass
@@ -471,16 +491,15 @@ class NexusDataSource(object):
                         nSpectra *= iDummy
                     if mcaselectiontype == "sum":
                         # sum already calculated
-                        if DEBUG:
-                            print("SUM")
+                        _logger.debug("SUM")
                     elif mcaselectiontype in ["avg", "average"]:
                         # calculate the average
-                        if DEBUG:
-                            print("AVERAGE")
+                        _logger.debug("AVERAGE")
                         data /= nSpectra
-                    else:                        
-                        print("Unsupported selection type %s" % mcaselectiontype)
-                        print("Calculating average")
+                    else:
+                        _logger.warning("Unsupported selection type %s",
+                                        mcaselectiontype)
+                        _logger.warning("Calculating average")
                         data /= nSpectra
                 elif len(data.shape) == 2:
                     if min(data.shape) == 1:
@@ -504,7 +523,7 @@ class NexusDataSource(object):
                 if len(selection[cnt]) > 1:
                     for xidx in range(1, len(selection[cnt])):
                         path =  entry + selection['cntlist'][selection[cnt][xidx]]
-                        data = phynxFile[path].value
+                        data = phynxFile[path][()]
                         output.x.append(data)
             elif cnt == 'm':
                 #only one monitor
