@@ -2,7 +2,7 @@
 #
 # The PyMca X-Ray Fluorescence Toolkit
 #
-# Copyright (c) 2018 European Synchrotron Radiation Facility
+# Copyright (c) 2018-2019 European Synchrotron Radiation Facility
 #
 # This file is part of the PyMca X-ray Fluorescence Toolkit developed at
 # the ESRF by the Software group.
@@ -34,8 +34,25 @@ import os
 from operator import itemgetter
 import re
 import posixpath
+try:
+    # try to import hdf5plugin
+    import hdf5plugin
+except:
+    # but do not crash just because of it
+    pass
 from h5py import File, Dataset, Group
-DEBUG = 0
+try:
+    from silx.io import is_dataset, is_group
+except:
+    def is_dataset(something):
+        return False
+
+    def is_group(something):
+        return False
+
+import logging
+
+_logger = logging.getLogger(__name__)
 
 #sorting method
 def h5py_sorting(object_list):
@@ -54,8 +71,7 @@ def h5py_sorting(object_list):
         posixNames = [item[1].name for item in object_list]
     except AttributeError:
         # Typical of broken external links
-        if DEBUG:
-            print("HDF5Widget: Cannot get posixNames")
+        _logger.debug("HDF5Widget: Cannot get posixNames")
         return object_list
 
     # This implementation only sorts entries
@@ -77,7 +93,7 @@ def h5py_sorting(object_list):
 
     try:
         if sorting_key != 'name':
-            sorting_list = [(o[1][sorting_key].value, o)
+            sorting_list = [(o[1][sorting_key][()], o)
                            for o in object_list]
             sorted_list = sorted(sorting_list, key=itemgetter(0))
             return [x[1] for x in sorted_list]
@@ -91,8 +107,8 @@ def h5py_sorting(object_list):
         #The only way to reach this point is to have different
         #structures among the different entries. In that case
         #defaults to the unfiltered case
-        print("WARNING: Default ordering")
-        print("Probably all entries do not have the key %s" % sorting_key)
+        _logger.warning("Default ordering. "
+                        "Probably all entries do not have the key %s", sorting_key)
         return object_list
 
 def _get_number_list(txt):
@@ -105,12 +121,16 @@ def isGroup(item):
         return True
     elif hasattr(item, "keys"):
         return True
+    elif is_group(item):
+        return True
     else:
         return False
 
 def isDataset(item):
     if isinstance(item, Dataset):
         return True
+    elif is_dataset(item):
+        return True
     else:
         return False
 
@@ -126,6 +146,63 @@ def getEntryName(path):
         candidate = posixpath.dirname(entry)
     return entry
 
+def getTitle(h5file, path):
+    """
+    Retrieve the title associated to the entry asoociated to the provided path
+    It returns an emptry string of not title is found
+    """
+    entry = h5file[getEntryName(path)]
+    title = ''
+    if "title" in entry:
+        title = entry["title"][()]
+        if hasattr(title, "dtype"):
+            _logger.warning("entry title should be a string not an array")
+            if hasattr(title, "__len__"):
+                if len(title) == 1:
+                    title = title[0]
+        if hasattr(title, "decode"):
+            title = title.decode("utf-8")
+    return title
+
+def getNXdataList(h5file, path, objects=False):
+    """
+    Retrieve the hdf5 group names down a given path where the NXclass attribute
+    is set to "NXdata".
+
+    If groups is False (default) it returns the dataset names.
+    If groups is True it returns the actual objects.
+    """
+    return getNXClassList(h5file, path, classes=["NXdata", b"NXdata"], objects=objects)
+
+def getNXClassList(h5file, path, classes, objects=False):
+    """
+    Retrieve the hdf5 group names down a given path where the NXclass attribute
+    is set to one of the items in the classes list.
+
+    If objects is False (default) it returns the group names.
+    If objects is True it returns the actual HDF5 group objects.
+    """
+    pathList =[]
+    def visit_function(name, obj):
+        if isGroup(obj):
+            append = False
+            forget = False
+            namebased = False
+            for key, value in obj.attrs.items():
+                if key in ["NX_class", b"NX_class"]:
+                    if value in classes:
+                        append = True
+                    else:
+                        forget = True
+            if append:
+                if objects:
+                    pathList.append(obj)
+                else:
+                    pathList.append(obj.name)
+    if hasattr(h5file[path], "visititems"):
+        # prevent errors dealing with toplevel datasets
+        h5file[path].visititems(visit_function)
+    return pathList
 
 def getMcaList(h5file, path, dataset=False, ignore=None):
     """
@@ -158,7 +235,7 @@ def getMcaList(h5file, path, dataset=False, ignore=None):
                   "energy"]
     datasetList =[]
     def visit_function(name, obj):
-        if isDataset(obj):
+        if is_dataset(obj):
             append = False
             forget = False
             namebased = False
@@ -421,7 +498,7 @@ def getInstrumentGroup(h5file, path):
         return None
     else:
         if n > 1:
-            print("WARNING: More than one instrument associated to the same entry")
+            _logger.warning("More than one instrument associated to the same entry")
         return groups[0]
 
 def getScannedPositioners(h5file, path):
@@ -460,19 +537,26 @@ def getScannedPositioners(h5file, path):
         if not len(scanned):
             entry = h5file[entry_name]
             if "title" in entry:
-                title = entry["title"].value
+                title = entry["title"][()]
+                if hasattr(title, "dtype"):
+                    _logger.warning("entry title should be a string not an array")
+                    if hasattr(title, "__len__"):
+                        if len(title) == 1:
+                            title = title[0]
                 if hasattr(title, "decode"):
                     title = title.decode("utf-8")
-                tokens = title.split()
-                candidates = [key for key, item in measurement.items() if \
-                                            isDataset(item) and \
-                                            (key in tokens)]
-                indices = []
-                for key in candidates:
-                    indices.append((tokens.index(key), key))
-                indices.sort()
-                if len(indices):
-                    scanned = [measurement[key].name for idx, key in indices]
+                if hasattr(title, "split"):
+                    tokens = title.split()
+                else:
+                    candidates = [key for key, item in measurement.items() if \
+                                                isDataset(item) and \
+                                                (key in tokens)]
+                    indices = []
+                    for key in candidates:
+                        indices.append((tokens.index(key), key))
+                    indices.sort()
+                    if len(indices):
+                        scanned = [measurement[key].name for idx, key in indices]
     return scanned
 
 if __name__ == "__main__":
@@ -483,14 +567,18 @@ if __name__ == "__main__":
     except:
         print("Usage: NexusTools <file> <key>")
         sys.exit()
-    h5 = h5py.File(sourcename, 'r')
+    try:
+        from silx.io import open as h5open
+        h5 = h5open(sourcename)
+    except:
+        h5 = h5py.File(sourcename, 'r')
     entries = getNXClassGroups(h5, "/", ["NXentry", b"NXentry"], single=False)
     if not len(entries):
         entries = [item for name, item in h5["/"].items() if isGroup(item)]
     for entry in entries:
         print("Entry name = %s" % entry.name)
         if "title" in entry:
-            print("Entry title = %s" % entry["title"].value)
+            print("Entry title = %s" % entry["title"][()])
         measurement = getMeasurementGroup(h5, entry.name)
         if measurement is None:
             print("No measurement")
