@@ -1,5 +1,5 @@
 #/*##########################################################################
-# Copyright (C) 2004-2018 V.A. Sole, ESRF - D. Dale CHESS
+# Copyright (C) 2004-2019 V.A. Sole, ESRF - D. Dale CHESS
 #
 # This file is part of the PyMca X-ray Fluorescence Toolkit developed at
 # the ESRF by the Software group.
@@ -34,7 +34,14 @@ import posixpath
 import gc
 import re
 from operator import itemgetter
-
+import logging
+_logger = logging.getLogger(__name__)
+try:
+    # try to import hdf5plugins
+    import hdf5plugin
+except:
+    # but do not crash just because of it
+    pass
 import h5py
 import weakref
 
@@ -48,7 +55,14 @@ except ImportError:
         return isinstance(node, h5py.Group)
 
     def h5open(filename):
-        return h5py.File(filename, "r")
+        try:
+            return h5py.File(filename, "r")
+        except OSError:
+            if h5py.version.hdf5_version_tuple < (1, 10):
+                # no reason to try SWMR mode
+                raise
+            _logger.info("Cannot open %s. Trying in SWMR mode" % filename)
+            return h5py.File(filename, "r", libver='latest', swmr=True)
 
 from PyMca5.PyMcaGui import PyMcaQt as qt
 safe_str = qt.safe_str
@@ -60,7 +74,7 @@ else:
         return x
 
 
-DEBUG = 0
+
 QVERSION = qt.qVersion()
 
 
@@ -81,8 +95,7 @@ def h5py_sorting(object_list):
         posixNames = [item[1].name for item in object_list]
     except AttributeError:
         # Typical of broken external links
-        if DEBUG:
-            print("HDF5Widget: Cannot get posixNames")
+        _logger.debug("HDF5Widget: Cannot get posixNames")
         return object_list
 
     # This implementation only sorts entries
@@ -104,7 +117,7 @@ def h5py_sorting(object_list):
 
     try:
         if sorting_key != 'name':
-            sorting_list = [(o[1][sorting_key].value, o)
+            sorting_list = [(o[1][sorting_key][()], o)
                            for o in object_list]
             sorted_list = sorted(sorting_list, key=itemgetter(0))
             return [x[1] for x in sorted_list]
@@ -123,7 +136,7 @@ def h5py_sorting(object_list):
         return object_list
 
 def _get_number_list(txt):
-    rexpr = '[/a-zA-Z:-]'
+    rexpr = '[/a-zA-Z:_-]'
     nbs= [float(w) for w in re.split(rexpr, txt) if w not in ['',' ']]
     return nbs
 
@@ -223,7 +236,7 @@ class H5NodeProxy(object):
                 except:
                     #one cannot afford any error, so I revert to the old
                     # method where values where used instead of items
-                    if DEBUG:
+                    if _logger.getEffectiveLevel() == logging.DEBUG:
                         raise
                     else:
                         # tmpList = list(self.getNode(self.name).values())
@@ -324,8 +337,8 @@ class H5NodeProxy(object):
                         self._type = _type
                         if _type in ["NXdata"]:
                             self._color = qt.QColor(qt.Qt.blue)
-                        elif _type in ["NXroot", "NXentry"] and ("default" in attrs):
-                            self._color = qt.QColor(qt.Qt.blue)                        
+                        elif ("default" in attrs):
+                            self._color = qt.QColor(qt.Qt.blue)
                         #self._attrs = attrs
                         break
                         #self._type = _type[2].upper() + _type[3:]
@@ -428,10 +441,10 @@ class FileModel(qt.QAbstractItemModel):
                                 if len(children[idx].getNode().shape):
                                     #stored as an array of strings!!!
                                     #return just the first item
-                                    return MyQVariant("%s" % children[idx].getNode().value[0])
+                                    return MyQVariant("%s" % children[idx].getNode()[()][0])
                                 else:
                                     #stored as a string
-                                    return MyQVariant("%s" % children[idx].getNode().value)
+                                    return MyQVariant("%s" % children[idx].getNode()[()])
                 return MyQVariant(item.type)
             if column == 2:
                 return MyQVariant(item.shape)
@@ -581,8 +594,7 @@ class FileModel(qt.QAbstractItemModel):
         self.sigFileAppended.emit(ddict)
 
     def clear(self):
-        if DEBUG:
-            print("Clear called")
+        _logger.debug("Clear called")
         # reset is considered obsolete under Qt 5.
         if hasattr(self, "reset"):
             self.reset()
@@ -763,7 +775,7 @@ class Hdf5SelectionDialog(qt.QDialog):
                     "Invalid itemtype %s, should be 'group', 'dataset' or 'any'" % itemtype)
 
         if filename is None:
-            filename = _getFilenameDialog()
+            filename = _getFilenameDialog(parent=parent)
         if filename is None:
             raise IOError("No filename specified")
 
@@ -835,14 +847,14 @@ class Hdf5SelectionDialog(qt.QDialog):
         return ret
 
 
-def _getFilenameDialog():
+def _getFilenameDialog(parent=None):
     """Open a dialog to select a file in a filesystem tree view.
     Return the selected filename."""
     from PyMca5.PyMcaGui.io import PyMcaFileDialogs
     fileTypeList = ['HDF5 Files (*.h5 *.nxs *.hdf)',
                     'HDF5 Files (*)']
     message = "Open HDF5 file"
-    filenamelist, ffilter = PyMcaFileDialogs.getFileList(parent=None,
+    filenamelist, ffilter = PyMcaFileDialogs.getFileList(parent=parent,
                                 filetypelist=fileTypeList,
                                 message=message,
                                 getfilter=True,
@@ -853,7 +865,7 @@ def _getFilenameDialog():
     return filenamelist[0]
 
 
-def getDatasetValueDialog(filename=None, message=None):
+def getDatasetValueDialog(filename=None, message=None, parent=None):
     """Open a dialog to select a dataset in a HDF5 file.
     Return the value of the dataset.
 
@@ -864,7 +876,7 @@ def getDatasetValueDialog(filename=None, message=None):
     :param str message: Message used as window title for dialog
     :return: HDF5 dataset as numpy array, or None
     """
-    hdf5Dialog = Hdf5SelectionDialog(None, filename, message,
+    hdf5Dialog = Hdf5SelectionDialog(parent, filename, message,
                                      "dataset")
     ret = hdf5Dialog.exec_()
     if not ret:
@@ -874,19 +886,44 @@ def getDatasetValueDialog(filename=None, message=None):
 
     with h5open(filename) as hdf5File:
         hdf5Item = hdf5File[selectedHdf5Uri.split("::")[-1]]
-        data = hdf5Item.value
+        data = hdf5Item[()]
 
     return data
 
+def getDatasetUri(parent=None, filename=None, message=None):
+    # TODO: Accept a filter for type of dataset
+    hdf5Dialog = Hdf5SelectionDialog(parent, filename, message, "dataset")
+    ret = hdf5Dialog.exec_()
+    if not ret:
+        return None
+    selectedHdf5Uri = hdf5Dialog.selectedItemUri
+    return selectedHdf5Uri
 
-def getDatasetDialog(filename=None, value=False, message=None):
+def getGroupUri(parent=None, filename=None, message=None):
+    # TODO: Accept a filter for a particular attribute (NXclass)
+    hdf5Dialog = Hdf5SelectionDialog(parent, filename, message, "dataset")
+    ret = hdf5Dialog.exec_()
+    if not ret:
+        return None
+    selectedHdf5Uri = hdf5Dialog.selectedItemUri
+    return selectedHdf5Uri
+
+def getUri(parent=None, filename=None, message=None):
+    hdf5Dialog = Hdf5SelectionDialog(parent, filename, message, "any")
+    ret = hdf5Dialog.exec_()
+    if not ret:
+        return None
+    selectedHdf5Uri = hdf5Dialog.selectedItemUri
+    return selectedHdf5Uri
+
+def getDatasetDialog(filename=None, value=False, message=None, parent=None):
     # function kept for backward compatibility, in case someone
     # uses it with value=False outside PyMca5
     if value:
-        return getDatasetValueDialog(filename, message)
+        return getDatasetValueDialog(filename=filename, message=message,
+                                     parent=parent)
 
-    hdf5Dialog = Hdf5SelectionDialog(None, filename, message,
-                                     "dataset")
+    hdf5Dialog = Hdf5SelectionDialog(parent, filename, message, "dataset")
     ret = hdf5Dialog.exec_()
     if not ret:
         return None
@@ -895,7 +932,7 @@ def getDatasetDialog(filename=None, value=False, message=None):
     return hdf5File[selectedHdf5Uri.split("::")[-1]]
 
 
-def getGroupNameDialog(filename=None, message=None):
+def getGroupNameDialog(filename=None, message=None, parent=None):
     """Open a dialog to select a group in a HDF5 file.
     Return the name of the group.
 
@@ -904,8 +941,7 @@ def getGroupNameDialog(filename=None, message=None):
     :param str message: Message used as window title for dialog
     :return: HDF5 group name
     """
-    hdf5Dialog = Hdf5SelectionDialog(None, filename, message,
-                                     "group")
+    hdf5Dialog = Hdf5SelectionDialog(parent, filename, message, "group")
     ret = hdf5Dialog.exec_()
     if not ret:
         return None
@@ -930,4 +966,6 @@ if __name__ == "__main__":
             print(phynxFile[ddict['name']].dtype, phynxFile[ddict['name']].shape)
     fileView.sigHDF5WidgetSignal.connect(mySlot)
     fileView.show()
-    sys.exit(app.exec_())
+    ret = app.exec_()
+    app = None
+    sys.exit(ret)
