@@ -1,5 +1,5 @@
 #/*##########################################################################
-# Copyright (C) 2004-2018 V.A. Sole, European Synchrotron Radiation Facility
+# Copyright (C) 2004-2019 V.A. Sole, European Synchrotron Radiation Facility
 #
 # This file is part of the PyMca X-ray Fluorescence Toolkit developed at
 # the ESRF by the Software group.
@@ -33,6 +33,7 @@ import posixpath
 import weakref
 import gc
 import h5py
+import logging
 
 from PyMca5.PyMcaGui import PyMcaQt as qt
 from PyMca5.PyMcaCore import NexusTools
@@ -57,7 +58,8 @@ from PyMca5.PyMcaIO import ConfigDict
 if "PyMcaDirs" in sys.modules:
     from PyMca5 import PyMcaDirs
 
-DEBUG=0
+_logger = logging.getLogger(__name__)
+
 
 class Buttons(qt.QWidget):
 
@@ -122,6 +124,7 @@ class QNexusWidget(qt.QWidget):
 
     def build(self):
         self.mainLayout = qt.QVBoxLayout(self)
+        self.mainLayout.setContentsMargins(5, 5, 5, 0)
         self.splitter = qt.QSplitter(self)
         self.splitter.setOrientation(qt.Qt.Vertical)
         self.hdf5Widget = HDF5Widget.HDF5Widget(self._defaultModel,
@@ -296,8 +299,7 @@ class QNexusWidget(qt.QWidget):
         else:
             ddict['HDF5'] ={'WidgetConfiguration':\
                              self.getWidgetConfiguration()}
-        if DEBUG:
-            print("TODO - Add selection options")
+        _logger.debug("TODO - Add selection options")
         ddict.write(fname)
 
     def _deleteAllCountersFromTable(self):
@@ -401,8 +403,7 @@ class QNexusWidget(qt.QWidget):
             if type(self._aliasList) == type(""):
                 self._aliasList = [ddict['aliases']]
         self.cntTable.build(self._cntList, self._aliasList)
-        if DEBUG:
-            print("TODO - Add selection options")
+        _logger.debug("TODO - Add selection options")
 
     def setDataSource(self, dataSource):
         self.data = dataSource
@@ -423,6 +424,10 @@ class QNexusWidget(qt.QWidget):
         self.hdf5Widget.setModel(model)
         for source in self.data._sourceObjectList:
             self.hdf5Widget.model().appendPhynxFile(source, weakreference=True)
+        if len(self.data._sourceObjectList) == 1:
+            # only one file, expand by default
+            if hasattr(self.hdf5Widget, "expandToDepth"):
+                self.hdf5Widget.expandToDepth(0)
 
     def setFile(self, filename):
         self._data = self.hdf5Widget.model().openFile(filename, weakreference=True)
@@ -444,7 +449,7 @@ class QNexusWidget(qt.QWidget):
             fileIndex = self.data.sourceName.index(filename)
             phynxFile  = self.data._sourceObjectList[fileIndex]
         else:
-            phynxFile  = h5py.File(filename, 'r')
+            phynxFile  = HDF5Widget.h5open(filename)
         info = self.getInfo(phynxFile, name)
         widget = HDF5Info.HDF5InfoWidget()
         widget.notifyCloseEventToWidget(self)
@@ -480,7 +485,7 @@ class QNexusWidget(qt.QWidget):
                     try:
                         widget.w.setDataset(dataset)
                     except:
-                        print("Error filling table")
+                        _logger.error("Error filling table")
                     widget.addTab(widget.w, 'DataView')
                     widget.setCurrentWidget(widget.w)
         elif Hdf5NodeView is not None:
@@ -500,6 +505,8 @@ class QNexusWidget(qt.QWidget):
             # handle a right click on a group or a dataset of string type
             _hdf5WidgetDatasetMenu.addAction(QString("Show Information"),
                                              self._showInfoWidgetSlot)
+            _hdf5WidgetDatasetMenu.addAction(QString("Copy Path to Clipboard"),
+                                             self._copyPathSlot)
             _hdf5WidgetDatasetMenu.exec_(qt.QCursor.pos())
         else:
             #handle a right click on a numeric dataset
@@ -515,6 +522,8 @@ class QNexusWidget(qt.QWidget):
             else:
                 _hdf5WidgetDatasetMenu.addAction(QString("Show Information"),
                                                  self._showInfoWidgetSlot)
+                _hdf5WidgetDatasetMenu.addAction(QString("Copy Path to Clipboard"),
+                                             self._copyPathSlot)
                 _hdf5WidgetDatasetMenu.exec_(qt.QCursor.pos())
         self._lastItemDict = None
         return
@@ -537,6 +546,15 @@ class QNexusWidget(qt.QWidget):
                                    ddict['name'],
                                    dset=is_numeric_dataset)
 
+    def _copyPathSlot(self, ddict=None):
+        if ddict is None:
+            ddict = self._lastItemDict
+        try:
+            clipboard = qt.QApplication.clipboard()
+            clipboard.setText(ddict["name"])
+        except:
+            _logger.warning("Unsuccessful copy to clipboard")
+
     def _openDataset(self, ddict=None):
         if ddict is None:
             ddict = self._lastItemDict
@@ -611,7 +629,8 @@ class QNexusWidget(qt.QWidget):
             scanned = []
             mcaList = []
             if posixpath.dirname(entryName) != entryName:
-                with h5py.File(ddict['file'], "r") as h5file:
+                h5file = HDF5Widget.h5open(ddict['file'])
+                try:
                     measurement = NexusTools.getMeasurementGroup(h5file,
                                                                  ddict['name'])
                     scanned = NexusTools.getScannedPositioners(h5file,
@@ -621,6 +640,10 @@ class QNexusWidget(qt.QWidget):
                                        if self._isNumeric(item)]
                     if self._mca:
                         mcaList = NexusTools.getMcaList(h5file, entryName)
+                finally:
+                    h5file.close()
+                    h5file = None
+                    del h5file
             for i in range(len(scanned)):
                 key = scanned[i]
                 cntList.append(key)
@@ -661,19 +684,16 @@ class QNexusWidget(qt.QWidget):
                     self.tableTab.insertTab(2, self.mcaTable, "MCA")
                 elif (len(mcaList)==0) and (nTabs > 2):
                     self.tableTab.removeTab(2)
-            if DEBUG:
-                print("currentTab = ", currentTab)
+            _logger.debug("currentTab = %s", currentTab)
             if currentTab != "USER":
                 if (len(mcaList) > 0) and (len(cntList) == 0):
                     idx = self.tableTabOrder.index("MCA")
                     self.tableTab.setCurrentIndex(idx)
-                    if DEBUG:
-                        print("setting tab = ", idx, "MCA")
+                    _logger.debug("setting tab = %s MCA", idx)
                 elif (len(mcaList) == 0) and (len(cntList) > 0):
                     idx = self.tableTabOrder.index("AUTO")
                     self.tableTab.setCurrentIndex(idx)
-                    if DEBUG:
-                        print("setting tab = ", idx, "AUTO")
+                    _logger.debug("setting tab = %s AUTO", idx)
             self._lastEntry = currentEntry
         if ddict['event'] == 'itemClicked':
             if ddict['mouse'] == "right":
@@ -697,8 +717,7 @@ class QNexusWidget(qt.QWidget):
                     else:
                         self.tableTab.setCurrentIndex(0)
                 if not self._isNumericType(ddict['dtype']):
-                    if DEBUG:
-                        print("string like %s" % ddict['dtype'])
+                    _logger.debug("string like %s", ddict['dtype'])
                 else:
                     root = ddict['name'].split('/')
                     root = "/" + root[1]
@@ -737,8 +756,7 @@ class QNexusWidget(qt.QWidget):
                         root = "/" + root[1]
                         cnt  = ddict['name'].split(root)[-1]
                         if cnt not in self._cntList:
-                            if DEBUG:
-                                print("USING SECOND WAY")
+                            _logger.debug("USING SECOND WAY")
                             self._cntList.append(cnt)
                             basename = posixpath.basename(cnt)
                             if basename not in self._aliasList:
@@ -747,36 +765,30 @@ class QNexusWidget(qt.QWidget):
                                 self._aliasList.append(cnt)
                             self.cntTable.build(self._cntList, self._aliasList)
                         return
-                if DEBUG:
-                    print("Unhandled item type: %s" % ddict['dtype'])
+                _logger.debug("Unhandled item type: %s", ddict['dtype'])
 
 
     def _addMcaAction(self):
-        if DEBUG:
-            print("_addMcaAction received")
+        _logger.debug("_addMcaAction received")
         self.mcaAction("ADD")
 
     def _removeMcaAction(self):
-        if DEBUG:
-            print("_removeMcaAction received")
+        _logger.debug("_removeMcaAction received")
         self.mcaAction("REMOVE")
 
     def _replaceMcaAction(self):
-        if DEBUG:
-            print("_replaceMcaAction received")
+        _logger.debug("_replaceMcaAction received")
         self.mcaAction("REPLACE")
 
     def mcaAction(self, action="ADD"):
-        if DEBUG:
-            print("mcaAction %s" % action)
+        _logger.debug("mcaAction %s", action)
         self.mcaTable.getMcaSelection()
         ddict = {}
         ddict['action'] = "%s MCA" % action
         self.buttonsSlot(ddict, emit=True)
 
     def _addAction(self):
-        if DEBUG:
-            print("_addAction received")
+        _logger.debug("_addAction received")
         # formerly we had action and selection type
         text = qt.safe_str(self.tableTab.tabText(self.tableTab.currentIndex()))
         if text.upper() == "MCA":
@@ -791,8 +803,7 @@ class QNexusWidget(qt.QWidget):
             self.buttonsSlot(ddict, emit=True)
 
     def _removeAction(self):
-        if DEBUG:
-            print("_removeAction received")
+        _logger.debug("_removeAction received")
         text = qt.safe_str(self.tableTab.tabText(self.tableTab.currentIndex()))
         if text.upper() == "MCA":
             self._removeMcaAction()
@@ -806,8 +817,7 @@ class QNexusWidget(qt.QWidget):
             self.buttonsSlot(ddict, emit=True)
 
     def _replaceAction(self):
-        if DEBUG:
-            print("_replaceAction received")
+        _logger.debug("_replaceAction received")
         text = qt.safe_str(self.tableTab.tabText(self.tableTab.currentIndex()))
         if text.upper() == "MCA":
             self._replaceMcaAction()
@@ -821,8 +831,7 @@ class QNexusWidget(qt.QWidget):
             self.buttonsSlot(ddict, emit=True)
 
     def _configurationChangedAction(self, ddict):
-        if DEBUG:
-            print("_configurationChangedAction received", ddict)
+        _logger.debug("_configurationChangedAction received %s", ddict)
         if ddict["3d"]:
             self.autoTable.set3DEnabled(True, emit=False)
             self.cntTable.set3DEnabled(True, emit=False)
@@ -834,8 +843,7 @@ class QNexusWidget(qt.QWidget):
             self.cntTable.set2DEnabled(False, emit=False)
 
     def _autoTableUpdated(self, ddict):
-        if DEBUG:
-            print("_autoTableUpdated(self, ddict) ", ddict)
+        _logger.debug("_autoTableUpdated(self, ddict) %s", ddict)
         text = qt.safe_str(self.tableTab.tabText(self.tableTab.currentIndex()))
         if text.upper() == "AUTO":
             actions = self.actions.getConfiguration()
@@ -846,8 +854,7 @@ class QNexusWidget(qt.QWidget):
                     self._replaceAction()
 
     def _userTableUpdated(self, ddict):
-        if DEBUG:
-            print("_userTableUpdated(self, ddict) ", ddict)
+        _logger.debug("_userTableUpdated(self, ddict) %s", ddict)
         text = qt.safe_str(self.tableTab.tabText(self.tableTab.currentIndex()))
         if text.upper() == "USER":
             actions = self.actions.getConfiguration()
@@ -858,8 +865,7 @@ class QNexusWidget(qt.QWidget):
                     self._replaceAction()
 
     def _mcaTableUpdated(self, ddict):
-        if DEBUG:
-            print("_mcaTableUpdated(self, ddict) ", ddict)
+        _logger.debug("_mcaTableUpdated(self, ddict) %s", ddict)
         text = qt.safe_str(self.tableTab.tabText(self.tableTab.currentIndex()))
         if text.upper() == "MCA":
             actions = self.actions.getConfiguration()
@@ -870,8 +876,7 @@ class QNexusWidget(qt.QWidget):
                     self._replaceAction()
 
     def buttonsSlot(self, ddict, emit=True):
-        if DEBUG:
-            print("buttonsSlot(self, ddict,emit=True)", ddict, "emit = ", emit)
+        _logger.debug("buttonsSlot(self, %s,emit=%s)", ddict, emit)
         if self.data is None:
             return
         action, selectionType = ddict['action'].split()
@@ -933,7 +938,10 @@ class QNexusWidget(qt.QWidget):
                 sel['SourceType'] = "HDF5"
                 fileIndex = self.data.sourceName.index(filename)
                 phynxFile  = self.data._sourceObjectList[fileIndex]
-                entryIndex = list(phynxFile["/"].keys()).index(entry[1:])
+                if entry == "/":
+                    entryIndex = 1
+                else:
+                    entryIndex = list(phynxFile["/"].keys()).index(entry[1:])
                 sel['Key']        = "%d.%d" % (fileIndex+1, entryIndex+1)
                 sel['legend']     = os.path.basename(sel['SourceName'][0])+\
                                     " " + posixpath.basename(entry) #it was sel['Key']
@@ -969,6 +977,40 @@ class QNexusWidget(qt.QWidget):
                                 # at least twoD dataset
                                 selectionType= "2D"
                                 sel['scanselection'] = False
+                    n_axes = len(sel['selection']['x'])
+                    if n_axes > 1:
+                        selectionType = "%dD" % len(sel['selection']['x'])
+                        selectionTypeDecided = False
+                        nAxesItems = 1
+                        if n_axes == len(actualDataset.shape):
+                            for xCnt in cntSelection['x']:
+                                if cntSelection['cntlist'][xCnt].startswith("/"):
+                                    xDatasetPath = posixpath.join(entry,
+                                                            cntSelection['cntlist'][xCnt][1:])
+                                else:
+                                    xDatasetPath = posixpath.join(entry,
+                                                            cntSelection['cntlist'][xCnt])
+                                nAxesItems *= phynxFile[xDatasetPath].size
+                            if nAxesItems == actualDataset.size:
+                                # we have an image with the associated dimensions
+                                selectionTypeDediced = True
+
+                        if selectionTypeDecided:
+                            pass
+                        elif n_axes == 2:
+                            try:
+                                from silx import version_info as silx_version
+                            except ImportError:
+                                silx_version = (0, 0, 0)
+                            if silx_version < (0, 11):
+                                # we have to use the 3D visualization
+                                selectionType = "3D"
+                            else:
+                                # we can afford a scatter view
+                                selectionType = "2D"
+                        else:
+                            selectionType = "%dD" % n_axes
+                        sel['scanselection'] = False
                     sel['mcaselection']  = False
                 elif selectionType.upper() == "MCA":
                     sel['scanselection'] = False
@@ -1057,16 +1099,17 @@ class QNexusWidget(qt.QWidget):
             if 'event' in ddict:
                 if ddict['event'] == "closeEventSignal":
                     if ddict['id'] in self._widgetDict:
-                        if DEBUG:
+                        if _logger.getEffectiveLevel() == logging.DEBUG:
                             try:
                                 widget = self._widgetDict[ddict['id']]
-                                print("DELETING %s" % widget.windowTitle())
+                                _logger.debug("DELETING %s", widget.windowTitle())
                             except:
                                 pass
                         del self._widgetDict[ddict['id']]
 
 if __name__ == "__main__":
     import sys
+    _logger.setLevel(logging.DEBUG)
     app = qt.QApplication(sys.argv)
     try:
         #this is to add the 3D buttons ...
@@ -1090,4 +1133,5 @@ if __name__ == "__main__":
     w.sigAddSelection.connect(addSelection)
     w.sigRemoveSelection.connect(removeSelection)
     w.sigReplaceSelection.connect(replaceSelection)
-    sys.exit(app.exec_())
+    ret = app.exec_()
+    sys.exit(ret)
