@@ -1,5 +1,5 @@
 #/*##########################################################################
-# Copyright (C) 2004-2016 V.A. Sole, European Synchrotron Radiation Facility
+# Copyright (C) 2004-2019 V.A. Sole, European Synchrotron Radiation Facility
 #
 # This file is part of the PyMca X-ray Fluorescence Toolkit developed at
 # the ESRF by the Software group.
@@ -29,6 +29,7 @@ __license__ = "MIT"
 __copyright__ = "European Synchrotron Radiation Facility, Grenoble, France"
 import sys
 import os
+import logging
 from PyMca5.PyMcaGui import PyMcaQt as qt
 from PyMca5.PyMcaMath.fitting import SimpleFitModule
 from . import SimpleFitConfigurationGui
@@ -36,7 +37,8 @@ from PyMca5.PyMcaMath.fitting import SimpleFitUserEstimatedFunctions
 from . import Parameters
 from PyMca5.PyMcaGui import PlotWindow
 
-DEBUG = 0
+_logger = logging.getLogger(__name__)
+
 
 class TopWidget(qt.QWidget):
     def __init__(self, parent=None):
@@ -221,14 +223,14 @@ class SimpleFitGui(qt.QWidget):
                 functionsfile= qt.safe_str(fn)
             if not len(functionsfile):
                 return
-        if DEBUG:
+
+        try:
             self.fitModule.importFunctions(functionsfile)
-        else:
-            try:
-                self.fitModule.importFunctions(functionsfile)
-            except:
-                qt.QMessageBox.critical(self, "ERROR",
-                                        "Function not imported")
+        except:
+            if _logger.getEffectiveLevel() == logging.DEBUG:
+                raise
+            qt.QMessageBox.critical(self, "ERROR",
+                                    "Function not imported")
 
         config = self.fitModule.getConfiguration()
         self.topWidget.setFunctions(config['fit']['functions'])
@@ -253,8 +255,7 @@ class SimpleFitGui(qt.QWidget):
                 SimpleFitConfigurationGui.SimpleFitConfigurationGui()
         self._configurationDialog.setSimpleFitInstance(self.fitModule)
         if not self._configurationDialog.exec_():
-            if DEBUG:
-                print("NOT UPDATING CONFIGURATION")
+            _logger.debug("NOT UPDATING CONFIGURATION")
             oldConfig = self.fitModule.getConfiguration()
             self._configurationDialog.setConfiguration(oldConfig)
             return
@@ -276,8 +277,7 @@ class SimpleFitGui(qt.QWidget):
             idx = newConfig['fit']['functions'].index(fname) + 1
         idx = self.topWidget.backgroundCombo.findText(fname)
         self.topWidget.backgroundCombo.setCurrentIndex(idx)
-        if DEBUG:
-            print("TABLE TO BE CLEANED")
+        _logger.debug("TABLE TO BE CLEANED")
         #self.estimate()
 
     def setFitFunction(self, fname):
@@ -322,7 +322,7 @@ class SimpleFitGui(qt.QWidget):
             self.setStatus()
             self.parametersTable.fillTableFromFit(self.fitModule.paramlist)
         except:
-            if DEBUG:
+            if _logger.getEffectiveLevel() == logging.DEBUG:
                 raise
             text = "%s:%s" % (sys.exc_info()[0], sys.exc_info()[1])
             msg = qt.QMessageBox(self)
@@ -331,7 +331,6 @@ class SimpleFitGui(qt.QWidget):
             msg.exec_()
             self.setStatus("Ready (after estimate error)")
 
-
     def setStatus(self, text=None):
         if text is None:
             text = "%s" % self.fitModule.getStatus()
@@ -345,7 +344,7 @@ class SimpleFitGui(qt.QWidget):
             values,chisq,sigma,niter,lastdeltachi = self.fitModule.startFit()
             self.setStatus()
         except:
-            if DEBUG:
+            if _logger.getEffectiveLevel() == logging.DEBUG:
                 raise
             text = "%s:%s" % (sys.exc_info()[0], sys.exc_info()[1])
             msg = qt.QMessageBox(self)
@@ -433,7 +432,7 @@ def test():
     return w
 
 if __name__=="__main__":
-    DEBUG = 0
+    _logger.setLevel(logging.DEBUG)
     app = qt.QApplication([])
     w = test()
     app.exec_()
