@@ -2,7 +2,7 @@
 #
 # The PyMca X-Ray Fluorescence Toolkit
 #
-# Copyright (c) 2004-2018 European Synchrotron Radiation Facility
+# Copyright (c) 2004-2019 European Synchrotron Radiation Facility
 #
 # This file is part of the PyMca X-ray Fluorescence Toolkit developed at
 # the ESRF by the Software group.
@@ -33,11 +33,12 @@ __copyright__ = "European Synchrotron Radiation Facility, Grenoble, France"
 import sys
 import os
 import copy
+import logging
 import numpy
 import traceback
 from PyMca5.PyMcaGui import PyMcaQt as qt
 from PyMca5.PyMcaPhysics import Elements
-from PyMca5.PyMcaGui import PlotWindow
+from PyMca5.PyMcaGui.plotting import PlotWindow
 ScanWindow = PlotWindow.PlotWindow
 
 if hasattr(qt, "QString"):
@@ -45,7 +46,8 @@ if hasattr(qt, "QString"):
 else:
     QString = str
 
-DEBUG = 0
+_logger = logging.getLogger(__name__)
+
 
 class MaterialEditor(qt.QWidget):
     def __init__(self, parent=None, name="Material Editor",
@@ -146,7 +148,7 @@ class MaterialEditor(qt.QWidget):
                 #no message?
                 error = 1
                 del Elements.Material[material]
-                if DEBUG:
+                if _logger.getEffectiveLevel() == logging.DEBUG:
                     raise
                 continue
         return error
@@ -174,11 +176,13 @@ class MaterialEditor(qt.QWidget):
                                              density=density, thickness=thickness, listoutput=False)
             addButton = False
             if self.graph is None:
+                # probably dead code (ScanWindow not imported)
                 self.graphDialog = qt.QDialog(self)
                 self.graphDialog.mainLayout = qt.QVBoxLayout(self.graphDialog)
                 self.graphDialog.mainLayout.setContentsMargins(0, 0, 0, 0)
                 self.graphDialog.mainLayout.setSpacing(0)
-                self.graph = ScanWindow.ScanWindow(self.graphDialog)
+                #self.graph = ScanWindow.ScanWindow(self.graphDialog)
+                self.graph = ScanWindow(self.graphDialog)
                 self.graphDialog.mainLayout.addWidget(self.graph)
                 self.graph._togglePointsSignal()
                 self.graph.graph.crossPicker.setEnabled(False)
@@ -216,11 +220,13 @@ class MaterialEditor(qt.QWidget):
                                                                  energy)
             addButton = False
             if self.graph is None:
+                # probably dead code (ScanWindow.ScanWindow not imported)
                 self.graphDialog = qt.QDialog(self)
                 self.graphDialog.mainLayout = qt.QVBoxLayout(self.graphDialog)
                 self.graphDialog.mainLayout.setContentsMargins(0, 0, 0, 0)
                 self.graphDialog.mainLayout.setSpacing(0)
-                self.graph = ScanWindow.ScanWindow(self.graphDialog)
+                #self.graph = ScanWindow.ScanWindow(self.graphDialog)
+                self.graph = ScanWindow(self.graphDialog)
                 self.graphDialog.mainLayout.addWidget(self.graph)
                 self.graph._togglePointsSignal()
                 self.graph.graph.crossPicker.setEnabled(False)
@@ -237,11 +243,11 @@ class MaterialEditor(qt.QWidget):
                                 replot=False)
             for legend in ['Compton', 'Photo','Total']:
                 self.graph.addCurve(energy, numpy.array(data[legend.lower()]),
-                                legend=legend,
-                                xlabel='Energy (keV)',
-                                ylabel='Mass Att. (cm2/g)',
-                                replace=False,
-                                replot=False)
+                                    legend=legend,
+                                    xlabel='Energy (keV)',
+                                    ylabel='Mass Att. (cm2/g)',
+                                    replace=False,
+                                    replot=False)
             self.graph.setActiveCurve(legend+' '+'Mass Att. (cm2/g)')
             self.graph.setGraphTitle(ddict['Comment'])
             if self.graphDialog is not None:
@@ -298,7 +304,14 @@ class MaterialComboBox(qt.QComboBox):
             (result, index) = self.ownValidator.validate(qstring,0)
         if result != self.ownValidator.Valid:
             text = str(qstring)
-            if text.endswith(" "):
+            if "%" in text:
+                msg =  qt.QMessageBox(self)
+                msg.setIcon(qt.QMessageBox.Critical)
+                msg.setText("Invalid Material Name '%s'\n" % text + \
+                            "It contains a % character.\n")
+                msg.exec_()
+                msg = qt.QMessageBox.No
+            elif text.endswith(" "):
                 msg =  qt.QMessageBox(self)
                 msg.setIcon(qt.QMessageBox.Critical)
                 msg.setText("Invalid Material Name '%s'\n" % text + \
@@ -390,6 +403,8 @@ class MaterialValidator(qt.QValidator):
 
     def validate(self, qstring, pos):
         text = str(qstring)
+        if "%" in text:
+            return (self.Invalid, pos)
         if text == '-':
             return (self.Valid, pos)
         try:
@@ -410,10 +425,12 @@ class MaterialValidator(qt.QValidator):
 class MaterialGUI(qt.QWidget):
     sigMaterialMassAttenuationSignal = qt.pyqtSignal(object)
     sigMaterialTransmissionSignal = qt.pyqtSignal(object)
-    def __init__(self, parent=None, name="New Material",default={},
+    def __init__(self, parent=None, name="New Material",default=None,
                  comments=True, height=10, toolmode=False):
         qt.QWidget.__init__(self, parent)
         self.setWindowTitle(name)
+        if default is None:
+            default = {}
         self._default = default
         self._setCurrentDefault()
         for key in default.keys():
@@ -500,7 +517,7 @@ class MaterialGUI(qt.QWidget):
             densityLabel.setText("Default Density (g/cm3):")
             densityLabel.setAlignment(qt.Qt.AlignVCenter)
             self.__densityLine  = qt.QLineEdit(grid)
-            validator = qt.QDoubleValidator(self.__densityLine)
+            validator = qt.CLocaleQDoubleValidator(self.__densityLine)
             self.__densityLine.setValidator(validator)
 
             self.__densityLine.setReadOnly(False)
@@ -511,7 +528,7 @@ class MaterialGUI(qt.QWidget):
             thicknessLabel.setText("Default  Thickness  (cm):")
             thicknessLabel.setAlignment(qt.Qt.AlignVCenter)
             self.__thicknessLine  = qt.QLineEdit(grid)
-            validator = qt.QDoubleValidator(self.__thicknessLine)
+            validator = qt.CLocaleQDoubleValidator(self.__thicknessLine)
             self.__thicknessLine.setValidator(validator)
 
             gridLayout.addWidget(self.__thicknessLine, 1, 1)
@@ -602,7 +619,7 @@ class MaterialGUI(qt.QWidget):
         densityLabel.setAlignment(qt.Qt.AlignVCenter)
         self.__densityLine  = qt.QLineEdit(grid)
         self.__densityLine.setText("1.0")
-        validator = qt.QDoubleValidator(self.__densityLine)
+        validator = qt.CLocaleQDoubleValidator(self.__densityLine)
         self.__densityLine.setValidator(validator)
         self.__densityLine.setReadOnly(False)
 
@@ -611,7 +628,7 @@ class MaterialGUI(qt.QWidget):
         thicknessLabel.setAlignment(qt.Qt.AlignVCenter)
         self.__thicknessLine  = qt.QLineEdit(grid)
         self.__thicknessLine.setText("0.1")
-        validator = qt.QDoubleValidator(self.__thicknessLine)
+        validator = qt.CLocaleQDoubleValidator(self.__thicknessLine)
         self.__thicknessLine.setValidator(validator)
         self.__thicknessLine.setReadOnly(False)
 
@@ -670,8 +687,12 @@ class MaterialGUI(qt.QWidget):
         self.__massAttButton.clicked.connect(self.__massAttSlot)
 
     def setCurrent(self, matkey0):
-        if DEBUG:"setCurrent(self, matkey0) ", matkey0
+        _logger.debug("setCurrent(self, matkey0=%s)", matkey0)
         matkey = Elements.getMaterialKey(matkey0)
+        if self._default == {}:
+            firstTime = True
+        else:
+            firstTime = False
         if matkey is not None:
             if self.__toolMode:
                 #make sure the material CANNOT be modified
@@ -692,10 +713,11 @@ class MaterialGUI(qt.QWidget):
             if self.__toolMode:
                 self.__nameLine.setText("%s" % matkey)
             self.__fillingValues = False
+        if firstTime:
+            self.__table.resizeColumnToContents(0)
 
     def _fillValues(self):
-        if DEBUG:
-            print("fillValues(self)")
+        _logger.debug("fillValues(self)")
         self.__fillingValues = True
         if self.__comments:
             self.__nameLine.setText("%s" % self._current['Comment'])
@@ -733,9 +755,8 @@ class MaterialGUI(qt.QWidget):
 
     # http://bugs.debian.org/cgi-bin/bugreport.cgi?bug=666503
     def _updateCurrent(self):
-        if DEBUG:
-            print("updateCurrent(self)")
-            print("self._current before = ", self._current)
+        _logger.debug("updateCurrent(self)")
+        _logger.debug("self._current before = %s", self._current)
 
         self._current['CompoundList']     = []
         self._current['CompoundFraction'] = []
@@ -755,8 +776,7 @@ class MaterialGUI(qt.QWidget):
                 self._current['CompoundFraction'].append(float(txt1))
         self.__densitySlot(silent=True)
         self.__thicknessSlot(silent=True)
-        if DEBUG:
-            print("self._current after = ", self._current)
+        _logger.debug("self._current after = %s", self._current)
 
     def __densitySlot(self, silent=False):
         try:
@@ -804,8 +824,7 @@ class MaterialGUI(qt.QWidget):
         self.sigMaterialMassAttenuationSignal.emit(ddict)
 
     def __nameLineSlot(self):
-        if DEBUG:
-            print("__nameLineSlot(self)")
+        _logger.debug("__nameLineSlot(self)")
         qstring = self.__nameLine.text()
         text = str(qstring)
         if self.__toolMode:
@@ -857,8 +876,7 @@ class MaterialGUI(qt.QWidget):
             return
         item = self.__table.item(row, col)
         if item is not None:
-            if DEBUG:
-                print("table item is None")
+            _logger.debug("table item is None")
             qstring = item.text()
         else:
             qstring = ""
@@ -950,4 +968,5 @@ if __name__ == "__main__":
     else:
         demo = MaterialEditor(toolmode=False)
     demo.show()
-    app.exec_()
+    ret  = app.exec_()
+    app = None
