@@ -1,5 +1,5 @@
 #/*##########################################################################
-# Copyright (C) 2004-2018 V.A. Sole, European Synchrotron Radiation Facility
+# Copyright (C) 2004-2019 V.A. Sole, European Synchrotron Radiation Facility
 #
 # This file is part of the PyMca X-ray Fluorescence Toolkit developed at
 # the ESRF by the Software group.
@@ -30,6 +30,7 @@ __copyright__ = "European Synchrotron Radiation Facility, Grenoble, France"
 import sys
 import os
 import numpy
+import logging
 from PyMca5.PyMcaGraph.ctools import pnpoly
 from . import RGBCorrelatorGraph
 from . import ColormapDialog
@@ -61,7 +62,8 @@ OVERLAY_DRAW = True
 
 DEFAULT_COLORMAP_INDEX = 2
 DEFAULT_COLORMAP_LOG_FLAG = False
-DEBUG = 0
+_logger = logging.getLogger(__name__)
+
 
 USE_PICKER = False
 
@@ -323,9 +325,8 @@ class MaskImageWidget(qt.QWidget):
                 self._profileSignalSlot(ddict)
 
     def _profileSignalSlot(self, ddict):
-        if DEBUG:
-            print("_profileSignalSLot, event = %s" % ddict['event'])
-            print("Received ddict = ", ddict)
+        _logger.debug("_profileSignalSLot, event = %s", ddict['event'])
+        _logger.debug("Received ddict = %s", ddict)
 
         if ddict['event'] in [None, "NONE"]:
             #Nothing to be made
@@ -361,7 +362,7 @@ class MaskImageWidget(qt.QWidget):
                              self._profileSelectionSlot)
                 self._profileSelectionWindow.sigReplaceClicked.connect(
                              self._profileSelectionSlot)
-            self._interpolate =  SpecfitFuns.interpol
+            self._interpolate = SpecfitFuns.interpol
             #if I do not return here and the user interacts with the graph while
             #the profileSelectionWindow is not shown, I get crashes under Qt 4.5.3 and MacOS X
             #when calling _getProfileCurve
@@ -577,8 +578,7 @@ class MaskImageWidget(qt.QWidget):
 
             if npoints == 1:
                 #all points are the same
-                if DEBUG:
-                    print("START AND END POINT ARE THE SAME!!")
+                _logger.debug("START AND END POINT ARE THE SAME!!")
                 return
 
             if width < 0:  # width = pixelwidth - 1
@@ -687,9 +687,8 @@ class MaskImageWidget(qt.QWidget):
                 newRow0 = 0.0
                 newRow1 = -(col1-col0) * sinalpha + (row1-row0) * cosalpha
 
-                if DEBUG:
-                    print("new X0 Y0 = %f, %f  " % (newCol0, newRow0))
-                    print("new X1 Y1 = %f, %f  " % (newCol1, newRow1))
+                _logger.debug("new X0 Y0 = %f, %f  ", newCol0, newRow0)
+                _logger.debug("new X1 Y1 = %f, %f  ", newCol1, newRow1)
 
                 tmpX   = numpy.linspace(newCol0, newCol1, npoints).astype(numpy.float)
                 rotMatrix = numpy.zeros((2,2), numpy.float)
@@ -697,19 +696,19 @@ class MaskImageWidget(qt.QWidget):
                 rotMatrix[0,1] = - sinalpha
                 rotMatrix[1,0] =   sinalpha
                 rotMatrix[1,1] =   cosalpha
-                if DEBUG:
+                if _logger.getEffectiveLevel() == logging.DEBUG:
                     #test if I recover the original points
-                    testX = numpy.zeros((2, 1) , numpy.float)
+                    testX = numpy.zeros((2, 1), numpy.float)
                     colRow = numpy.dot(rotMatrix, testX)
-                    print("Recovered X0 = %f" % (colRow[0,0] + col0))
-                    print("Recovered Y0 = %f" % (colRow[1,0] + row0))
-                    print("It should be = %f, %f" % (col0, row0))
-                    testX[0,0] = newCol1
-                    testX[1,0] = newRow1
+                    _logger.debug("Recovered X0 = %f", colRow[0, 0] + col0)
+                    _logger.debug("Recovered Y0 = %f", colRow[1, 0] + row0)
+                    _logger.debug("It should be = %f, %f", col0, row0)
+                    testX[0, 0] = newCol1
+                    testX[1, 0] = newRow1
                     colRow = numpy.dot(rotMatrix, testX)
-                    print("Recovered X1 = %f" % (colRow[0,0] + col0))
-                    print("Recovered Y1 = %f" % (colRow[1,0] + row0))
-                    print("It should be = %f, %f" % (col1, row1))
+                    _logger.debug("Recovered X1 = %f", colRow[0, 0] + col0)
+                    _logger.debug("Recovered Y1 = %f", colRow[1, 0] + row0)
+                    _logger.debug("It should be = %f, %f", col1, row1)
 
                 #find the drawing limits
                 testX = numpy.zeros((2, 4) , numpy.float)
@@ -727,21 +726,18 @@ class MaskImageWidget(qt.QWidget):
 
                 for a in rowLimits0:
                     if (a >= shape[0]) or (a < 0):
-                        if DEBUG:
-                            print("outside row limits",a)
+                        _logger.debug("outside row limits %s" % a)
                         return
                 for a in colLimits0:
                     if (a >= shape[1]) or (a < 0):
-                        if DEBUG:
-                            print("outside column limits",a)
+                        _logger.debug("outside column limits %s" % a)
                         return
 
                 r0 = rowLimits0[0]
                 r1 = rowLimits0[1]
 
                 if r0 > r1:
-                    if DEBUG:
-                        print("r0 > r1", r0, r1)
+                    _logger.debug("r0 > r1 %s %s" % (r0, r1))
                     raise ValueError("r0 > r1")
 
                 x = numpy.zeros((2, npoints) , numpy.float)
@@ -831,8 +827,7 @@ class MaskImageWidget(qt.QWidget):
                                     float(deltaRow) * deltaRow)/(npoints-1.0)
                 xdata *= deltaDistance
         else:
-            if DEBUG:
-                print("Mode %s not supported yet" % ddict['mode'])
+            _logger.debug("Mode %s not supported yet %s", ddict['mode'])
             return
 
         self.__lastOverlayWidth = ddict['pixelwidth']
@@ -842,8 +837,7 @@ class MaskImageWidget(qt.QWidget):
         return xdata, ydata, legend, info
 
     def _profileSelectionSlot(self, ddict):
-        if DEBUG:
-            print(ddict)
+        _logger.debug("%s", ddict)
         # the curves as [[x0, y0, legend0, info0], ...]
         curveList = ddict['curves']
         label = ddict['label']
@@ -988,15 +982,13 @@ class MaskImageWidget(qt.QWidget):
                                 self._replaceImageClicked)
 
     def _setEraseSelectionMode(self):
-        if DEBUG:
-            print("_setEraseSelectionMode")
+        _logger.debug("_setEraseSelectionMode")
         self.__eraseMode = True
         self.__brushMode = True
         self.graphWidget.graph.setDrawModeEnabled(False)
 
     def _setRectSelectionMode(self):
-        if DEBUG:
-            print("_setRectSelectionMode")
+        _logger.debug("_setRectSelectionMode")
         self.__eraseMode = False
         self.__brushMode = False
         self.graphWidget.graph.setDrawModeEnabled(True,
@@ -1011,15 +1003,13 @@ class MaskImageWidget(qt.QWidget):
                                                   label="mask")
 
     def _setBrushSelectionMode(self):
-        if DEBUG:
-            print("_setBrushSelectionMode")
+        _logger.debug("_setBrushSelectionMode")
         self.__eraseMode = False
         self.__brushMode = True
         self.graphWidget.graph.setDrawModeEnabled(False)
 
     def _setBrush(self):
-        if DEBUG:
-            print("_setBrush")
+        _logger.debug("_setBrush")
         if self.__brushMenu is None:
             self.__brushMenu = qt.QMenu()
             self.__brushMenu.addAction(QString(" 1 Image Pixel Width"),
@@ -1088,7 +1078,7 @@ class MaskImageWidget(qt.QWidget):
             self.graphWidget.graph.setDrawModeEnabled(True,
                                                       'rectangle',
                                                       label='mask')
-            self.__brushMode  = False
+            self.__brushMode = False
             self.graphWidget.hideProfileSelectionIcons()
             self.graphWidget.selectionToolButton.setChecked(True)
             self.graphWidget.selectionToolButton.setDown(True)
@@ -1169,8 +1159,7 @@ class MaskImageWidget(qt.QWidget):
         self._resetSelection(True)
 
     def _resetSelection(self, owncall=True):
-        if DEBUG:
-            print("_resetSelection")
+        _logger.debug("_resetSelection")
         self.__selectionMask = None
         if self.__imageData is None:
             return
@@ -1210,7 +1199,7 @@ class MaskImageWidget(qt.QWidget):
         if data is None:
             self.__imageData = data
             self.__selectionMask = None
-            self.plotImage(update = True)
+            self.plotImage(update=True)
             self.graphWidget._zoomReset(replot=True)
             return
         else:
@@ -1519,7 +1508,8 @@ class MaskImageWidget(qt.QWidget):
 
     def __applyMaskToImage(self):
         if self.__selectionMask is None:
-            return
+            self.__selectionMask = numpy.zeros(self.__imageData.shape,
+                                               numpy.uint8)
         #if not self.__selectionFlag:
         #    print("Return because of selection flag")
         #    return
@@ -1530,23 +1520,20 @@ class MaskImageWidget(qt.QWidget):
                 self.__pixmap[self.__selectionMask == i] = color
             return
         if self._maxNRois < 2:
-            alteration = (1 - (0.2 * self.__selectionMask))
+            alteration = (1 - (0.3 * (self.__selectionMask > 0)))
         else:
             alteration = (1 - (0.2 * (self.__selectionMask > 0))) - \
                          0.1 * (self.__selectionMask == self._roiTags[self._nRoi - 1])
         if self.colormap is None:
-            if DEBUG:
-                print("Colormap is None")
+            _logger.debug("Colormap is None")
             if self.__image is not None:
                 if self.__image.format() == qt.QImage.Format_ARGB32:
-                    if DEBUG:
-                        print("__applyMaskToImage CASE 1")
+                    _logger.debug("__applyMaskToImage CASE 1")
                     for i in range(4):
                         self.__pixmap[:,:,i]  = (self.__pixmap0[:,:,i] *\
                                 alteration).astype(numpy.uint8)
                 else:
-                    if DEBUG:
-                        print("__applyMaskToImage CASE 2")
+                    _logger.debug("__applyMaskToImage CASE 2")
                     self.__pixmap = self.__pixmap0.copy()
                     tmp = self.__selectionMask > 0
                     self.__pixmap[tmp, 0] = 0x40
@@ -1559,8 +1546,7 @@ class MaskImageWidget(qt.QWidget):
                         self.__pixmap[roiTag, 3] = 2*0x40
             else:
                 if self.__defaultColormap > 1:
-                    if DEBUG:
-                        print("__applyMaskToImage CASE 3")
+                    _logger.debug("__applyMaskToImage CASE 3")
                     self.__pixmap = self.__pixmap0.copy()
                     for i in range(3):
                         self.__pixmap[:,:,i]  = (self.__pixmap0[:,:,i] *\
@@ -1573,8 +1559,7 @@ class MaskImageWidget(qt.QWidget):
                             for i in range(3):
                                 self.__pixmap[:,:,i] *= tmpMask
                 else:
-                    if DEBUG:
-                        print("__applyMaskToImage CASE 4")
+                    _logger.debug("__applyMaskToImage CASE 4")
                     self.__pixmap = self.__pixmap0.copy()
                     self.__pixmap[self.__selectionMask>0,0]    = 0x40
                     self.__pixmap[self.__selectionMask>0,2]    = 0x70
@@ -1594,10 +1579,26 @@ class MaskImageWidget(qt.QWidget):
                             self.__pixmap[tmpMask,2]    = 0xff
                             self.__pixmap[tmpMask,3]    = 0xff
         elif int(str(self.colormap[0])) > 1:     #color
-            if DEBUG:
-                print("__applyMaskToImage CASE 5")
+            _logger.debug("__applyMaskToImage CASE 5")
+            # default color should be black, pink or green
+            if int(str(self.colormap[0])) == 2:
+                # expected to be temp, use black
+                color = [0x00, 0x00, 0x00, 0xff]
+            elif int(str(self.colormap[0])) == 3:
+                # expected to be red, use green
+                color = [0x00, 0xff, 0x00, 0xff]
+            elif int(str(self.colormap[0])) == 4:
+                # expected to be green, use pink
+                color = [0xff, 0x66, 0xff, 0xff]
+            elif int(str(self.colormap[0])) == 5:
+                # expected to be blue, use yellow
+                color = [0xff, 0xff, 0x00, 0xff]
+            else:
+                color = [0x00, 0x00, 0x00, 0xff]
+
             for i in range(3):
-                self.__pixmap[:,:,i]  = (self.__pixmap0[:,:,i] * alteration)
+                self.__pixmap[:,:,i] = (self.__pixmap0[:,:,i] * alteration) + \
+                                       (1 - alteration) * color[i]
             if 0:
                 tmpMask = numpy.isfinite(self.__imageData)
                 goodData = numpy.isfinite(self.__imageData).min()
@@ -1606,8 +1607,7 @@ class MaskImageWidget(qt.QWidget):
                         for i in range(3):
                             self.__pixmap[:,:,i] *= tmpMask
         elif self._maxNRois > 1:
-            if DEBUG:
-                print("__applyMaskToImage CASE 6")
+            _logger.debug("__applyMaskToImage CASE 6")
             tmp  = 1 - (self.__selectionMask>0)
             tmp2 = (self.__selectionMask == self._roiTags[self._nRoi - 1])
             self.__pixmap[:, :, 2] = (0x70 * (self.__selectionMask>0) + \
@@ -1616,14 +1616,11 @@ class MaskImageWidget(qt.QWidget):
             self.__pixmap[:,:, 3] = (0x40 * (self.__selectionMask>0)   + 0x40 * tmp2) +\
                                       tmp * self.__pixmap0[:,:,3]
         else:
-            if DEBUG:
-                print("__applyMaskToImage CASE 7")
-            self.__pixmap = self.__pixmap0.copy()
-            tmp  = 1 - self.__selectionMask
-            self.__pixmap[:, :, 2] = (0x70 * self.__selectionMask) +\
-                                  tmp * self.__pixmap0[:,:,2]
-            self.__pixmap[:, :, 3] = (0x40 * self.__selectionMask) +\
-                                  tmp * self.__pixmap0[:,:,3]
+            _logger.debug("__applyMaskToImage CASE 7")
+            color = numpy.array([0xff, 0x66, 0xff, 0xff], dtype=numpy.uint8)
+            for i in range(3):
+                self.__pixmap[:,:,i] = (self.__pixmap0[:,:,i] * alteration) +\
+                            (1 - alteration) * color[i]
             if 0:
                 tmpMask = ~numpy.isfinite(self.__imageData)
                 badData = numpy.isfinite(self.__imageData).max()
@@ -1802,7 +1799,7 @@ class MaskImageWidget(qt.QWidget):
         elif self.__pixmap0 is not None:
             imageShape = self.__pixmap0.shape[0:2]
         else:
-            print("Cannot handle polygon mask")
+            _logger.warning("Cannot handle polygon mask")
             return
         x = self._xScale[0] + self._xScale[1] * numpy.arange(imageShape[1])
         y = self._yScale[0] + self._yScale[1] * numpy.arange(imageShape[0])
@@ -1888,8 +1885,7 @@ class MaskImageWidget(qt.QWidget):
             if ownsignal:
                 pass
             if None in [ddict['x'], ddict['y']]:
-                if DEBUG:
-                    print("Signal from outside region", ddict)
+                _logger.debug("Signal from outside region %s", ddict)
                 return
 
             if self.graphWidget.infoWidget.isHidden() or self.__brushMode:
@@ -1982,8 +1978,7 @@ class MaskImageWidget(qt.QWidget):
         self.sigMaskImageWidgetSignal.emit(ddict)
 
     def _zoomResetSignal(self):
-        if DEBUG:
-            print("_zoomResetSignal")
+        _logger.debug("_zoomResetSignal")
         self.graphWidget._zoomReset(replot=False)
         self.plotImage(True)
 
@@ -1996,7 +1991,8 @@ class MaskImageWidget(qt.QWidget):
         filedialog.setFileMode(filedialog.AnyFile)
         filedialog.setAcceptMode(qt.QFileDialog.AcceptSave)
         filedialog.setWindowIcon(qt.QIcon(qt.QPixmap(IconDict["gioconda16"])))
-        formatlist = ["ASCII Files *.dat",
+        formatlist = ["TIFF Files *.tif",
+                      "ASCII Files *.dat",
                       "EDF Files *.edf",
                       'CSV(, separated) Files *.csv',
                       'CSV(; separated) Files *.csv',
@@ -2008,7 +2004,7 @@ class MaskImageWidget(qt.QWidget):
         for f in formatlist:
                 strlist.append(f)
         if self._saveFilter is None:
-            self._saveFilter =formatlist[0]
+            self._saveFilter = formatlist[0]
         if hasattr(filedialog, "setFilters"):
             filedialog.setFilters(strlist)
             filedialog.selectFilter(self._saveFilter)
@@ -2027,12 +2023,12 @@ class MaskImageWidget(qt.QWidget):
                 self._saveFilter = qt.safe_str(filedialog.selectedFilter())
             else:
                 self._saveFilter = qt.safe_str(filedialog.selectedNameFilter())
-            filterused = "."+self._saveFilter[-3:]
+            filterused = "." + self._saveFilter[-3:]
             PyMcaDirs.outputDir = os.path.dirname(filename)
             if len(filename) < 4:
-                filename = filename+ filterused
+                filename = filename + filterused
             elif filename[-4:] != filterused :
-                filename = filename+ filterused
+                filename = filename + filterused
         else:
             filename = ""
         return filename
@@ -2063,10 +2059,15 @@ class MaskImageWidget(qt.QWidget):
             return
         if filename is None:
             filename = self.getOutputFileName()
-            if not len(filename):return
+            if not len(filename):
+                return
 
         if filename.lower().endswith(".edf"):
             ArraySave.save2DArrayListAsEDF(imageList, filename, labels)
+        elif filename.lower().endswith(".tif"):
+            ArraySave.save2DArrayListAsMonochromaticTiff(imageList,
+                                                         filename,
+                                                         labels)
         elif filename.lower().endswith(".csv"):
             if "," in self._saveFilter:
                 csvseparator = ","
