@@ -1,5 +1,5 @@
 #/*##########################################################################
-# Copyright (C) 2004-2018 V.A. Sole, European Synchrotron Radiation Facility
+# Copyright (C) 2004-2019 European Synchrotron Radiation Facility
 #
 # This file is part of the PyMca X-ray Fluorescence Toolkit developed at
 # the ESRF by the Software group.
@@ -40,6 +40,7 @@ import time
 import traceback
 import numpy
 from numpy import argsort, nonzero, take
+
 from . import LegendSelector
 from .ObjectPrintConfigurationDialog import ObjectPrintConfigurationDialog
 from . import McaROIWidget
@@ -226,8 +227,9 @@ class PlotWindow(PlotWidget.PlotWidget):
         self.additionalIcon     = qt.QIcon(qt.QPixmap(IconDict["additionalselect"]))
         self.polygonIcon = qt.QIcon(qt.QPixmap(IconDict["polygon"]))
 
-        self.printIcon	= qt.QIcon(qt.QPixmap(IconDict["fileprint"]))
-        self.saveIcon	= qt.QIcon(qt.QPixmap(IconDict["filesave"]))
+        self.printIcon = qt.QIcon(qt.QPixmap(IconDict["fileprint"]))
+        self.copyIcon = qt.QIcon(qt.QPixmap(IconDict["clipboard"]))
+        self.saveIcon = qt.QIcon(qt.QPixmap(IconDict["filesave"]))
 
         self.pluginIcon     = qt.QIcon(qt.QPixmap(IconDict["plugin"]))
 
@@ -434,7 +436,12 @@ class PlotWindow(PlotWidget.PlotWidget):
                                 self._subtractIconSignal,
                                 'Subtract Active Curve')
 
-        #save
+        # clipboard
+        self.copyToolButton = self._addToolButton(self.copyIcon,
+                                                  self._copyIconSignal,
+                                                  "Copy graph to clipboard")
+
+        # save
         infotext = 'Save Active Curve or Widget'
         tb = self._addToolButton(self.saveIcon,
                                  self._saveIconSignal,
@@ -707,6 +714,9 @@ class PlotWindow(PlotWidget.PlotWidget):
                     self.updateActiveImageColormap)
         self.colormapDialog._update()
 
+    def _copyIconSignal(self):
+        self.copyToClipboard()
+
     def updateActiveImageColormap(self, colormap, replot=True):
         if len(colormap) == 1:
             colormap = colormap[0]
@@ -772,11 +782,13 @@ class PlotWindow(PlotWidget.PlotWidget):
             self.roiDockWidget.setWindowTitle(self.windowTitle()+(" ROI"))
             # initialize with the ICR
             self._roiSignal({'event': "AddROI"})
-
-        if self.roiDockWidget.isHidden():
-            self.roiDockWidget.show()
+            self.roiDockWidget.raise_()
         else:
-            self.roiDockWidget.hide()
+            if self.roiDockWidget.isHidden():
+                self.roiDockWidget.show()
+                self.roiDockWidget.raise_()
+            else:
+                self.roiDockWidget.hide()
 
     def changeGridLevel(self):
         self.gridLevel += 1
@@ -1175,6 +1187,8 @@ class PlotWindow(PlotWidget.PlotWidget):
             todata   = ddict['roi']['to']
             self.removeMarker('ROI min')
             self.removeMarker('ROI max')
+            if self._middleROIMarkerFlag:
+                self.removeMarker('ROI middle')
             if ddict['key'] == 'ICR':
                 draggable = False
                 color = 'black'
@@ -1474,7 +1488,10 @@ class PlotWindow(PlotWidget.PlotWidget):
             os.remove(filename)
         if filterused[0].upper() == "WIDGET":
             fformat = filename[-3:].upper()
-            pixmap = qt.QPixmap.grabWidget(self)
+            if hasattr(qt.QPixmap,"grabWidget"):
+                pixmap = qt.QPixmap.grabWidget(self)
+            else:
+                pixmap = self.grab()
             if not pixmap.save(filename, fformat):
                 msg = qt.QMessageBox(self)
                 msg.setIcon(qt.QMessageBox.Critical)
@@ -1599,7 +1616,7 @@ class PlotWindow(PlotWidget.PlotWidget):
         ret = outfile.exec_()
         if not ret:
             return None
-        if hasattr(outfile, "selectredFilter"):
+        if hasattr(outfile, "selectedFilter"):
             outputFilter = qt.safe_str(outfile.selectedFilter())
         else:
             outputFilter = qt.safe_str(outfile.selectedNameFilter())
@@ -1640,8 +1657,12 @@ if __name__ == "__main__":
     backend = None
     if ("opengl" in sys.argv) or ("gl" in sys.argv) or ("OpenGL" in sys.argv):
         backend = "opengl"
-    elif "pyqtgraph" in sys.argv:
-        backend = "pyqtgraph"
+    elif "silx" in sys.argv:
+        backend = "silx"
+    elif "silx-gl" in sys.argv:
+        backend = "silx-gl"
+    else:
+        backend = "matplotlib"
     plot = PlotWindow(backend=backend, roi=True, control=True,
                           position=True, colormap=True)#uselegendmenu=True)
     plot.setPanWithArrowKeys(True)
@@ -1662,3 +1683,4 @@ if __name__ == "__main__":
     #plot.addCurve(x, 2 * y, "dummy 2")
     #print("All curves = ",   plot.getAllCurves())
     app.exec_()
+    app = None
