@@ -1,5 +1,5 @@
 #/*##########################################################################
-# Copyright (C) 2004-2018 V.A. Sole, European Synchrotron Radiation Facility
+# Copyright (C) 2004-2019 V.A. Sole, European Synchrotron Radiation Facility
 #
 # This file is part of the PyMca X-ray Fluorescence Toolkit developed at
 # the ESRF by the Software group.
@@ -138,6 +138,57 @@ plugin = [
 ":::::::::::   ::::::::::::::::::"
 ]
 
+clipboard = [
+"30 30 17 1 ",
+"  c #4F4E50",
+". c #6E6E70",
+"X c #F7941E",
+"o c #F89A2B",
+"O c #F8A038",
+"+ c #F8A848",
+"@ c #FAB86A",
+"# c #FAC17C",
+"$ c #7E7F80",
+"% c #919192",
+"& c #AEAEAF",
+"* c #FBCA8E",
+"= c #FCD6A8",
+"- c #C0BFC0",
+"; c #CECECF",
+": c #FEEDD9",
+"> c None",
+">>>>>>>>>>>>>>>>>>>>>>>>>>>>>>",
+">>>>>>>>>>>>>>>>>>>>>>>>>>>>>>",
+">>>>>>>>>>>>>>#@>>>>>>>>>>>>>>",
+">>>>>>>>>>>>>>Xo>>>>>>>>>>>>>>",
+">>>>>>>>>>>>>:XX:>>>>>>>>>>>>>",
+">>>>>>>>>>>>>@*=@>>>>>>>>>>>>>",
+">>>>>>>>>>>>:+>>@=>>>>>>>>>>>>",
+">>>>>>>::::>Oo:>+o>::::>>>>>>>",
+">>>>>>#XoX@*XXXoXX*@XXX@>>>>>>",
+">>>>>>oo+@:XXXXXXXX:@+oX>>>>>>",
+">>>>>>XO>>>========:>>+X>>>>>>",
+">>>>>>XO>>>>>>>>>>>>>>+X>>>>>>",
+">>>>>>XO>>>>>>>>>>>>>>+X:>>>>>",
+">>>>>>XO>>;;;;:;;-&;>>+X>>>>>>",
+">>>>>>XO>>$.    ..%;>>+X>>>>>>",
+">>>>>>XO>>>>>>>>>>>>>>OX>>>>>>",
+">>>>>>XO>>>;;;----&>>>+X:>>>>>",
+">>>>>>XO>>.       .>>>+X>>>>>>",
+">>>>>>XO>>;:>>>>>>>>>>+X>>>>>>",
+">>>>>>XO>>>;&:>>>>>>>>+X>>>>>>",
+">>>>>>XO>>%%%>>>>>>>>>+X:>>>>>",
+">>>>>>XO>>>>>>>>>>>>>>+X>>>>>>",
+">>>>>>XO>>;........&>>+X>>>>>>",
+">>>>>>XO>>%%&%&&&%%;>>+X>>>>>>",
+">>>>>>XO>>>>>>>>>>>>>>+X>>>>>>",
+">>>>>>XO>>>>>>>>>>>>>>+X>>>>>>",
+">>>>>>XO>>>>>>>>>>>>>>+X>>>>>>",
+">>>>>>oXooooooooooooooXX:>>>>>",
+">>>>>>=OooooooooooooooO#>>>>>>",
+">>>>>>>>>>>>>>>>>>>>>>>>>>>>>>"
+]
+
 crop =[
 "16 16 2 1",
 "   c none",
@@ -3165,8 +3216,168 @@ unselected = [
 "XXXXXXXXXXXXXXXX"
 ]
 
-
-
+bliss = [
+"24 24 135 2 ",
+"   c #272A7C",
+".  c #282B7D",
+"X  c #292C7D",
+"o  c #2B2E7D",
+"O  c #2C2F7E",
+"+  c #2D2F7E",
+"@  c #2E317E",
+"#  c #2F327F",
+"$  c #31347F",
+"%  c #46AC32",
+"&  c #46AC33",
+"*  c #4DAE3C",
+"=  c #3E914C",
+"-  c #50AF40",
+";  c #51AF40",
+":  c #51AF42",
+">  c #53AF44",
+",  c #52B043",
+"<  c #56B047",
+"1  c #59B14B",
+"2  c #60B257",
+"3  c #63B35A",
+"4  c #63B25B",
+"5  c #65B45A",
+"6  c #66B55A",
+"7  c #66B55B",
+"8  c #68B560",
+"9  c #6BB366",
+"0  c #6EB864",
+"q  c #70B36F",
+"w  c #72B969",
+"e  c #74B96F",
+"r  c #72B371",
+"t  c #73B472",
+"y  c #78B47B",
+"u  c #79B67A",
+"i  c #79B27C",
+"p  c #7BB37F",
+"a  c #85C17E",
+"s  c #313480",
+"d  c #313481",
+"f  c #333580",
+"g  c #373A81",
+"h  c #363882",
+"j  c #3B3E83",
+"k  c #383B84",
+"l  c #3D4085",
+"z  c #3E4085",
+"x  c #424484",
+"c  c #424486",
+"v  c #434587",
+"b  c #454986",
+"n  c #454789",
+"m  c #464788",
+"M  c #494B8B",
+"N  c #4F508C",
+"B  c #4E508D",
+"V  c #53578B",
+"C  c #53568D",
+"Z  c #595A91",
+"A  c #595B91",
+"S  c #59608D",
+"D  c #5A628D",
+"F  c #5E678E",
+"G  c #696A9C",
+"H  c #697891",
+"J  c #6E7F94",
+"K  c #707B97",
+"L  c #727C99",
+"P  c #737F98",
+"I  c #6B6DA0",
+"U  c #7375A6",
+"Y  c #7476A6",
+"T  c #797AA3",
+"R  c #7A7CAB",
+"E  c #7B7DAC",
+"W  c #7D7FAE",
+"Q  c #6E8192",
+"!  c #758997",
+"~  c #778897",
+"^  c #768997",
+"/  c #78809C",
+"(  c #789592",
+")  c #7B9097",
+"_  c #7B9694",
+"`  c #7C9498",
+"'  c #7D9598",
+"]  c #79A48A",
+"[  c #7FAD8A",
+"{  c #7CB380",
+"}  c #7FB584",
+"|  c #7E80AE",
+" . c #7F81AF",
+".. c #82979E",
+"X. c #80B088",
+"o. c #86A599",
+"O. c #92B59E",
+"+. c #95B89E",
+"@. c #8289A7",
+"#. c #8686AB",
+"$. c #8687AE",
+"%. c #8588AC",
+"&. c #8C8DAF",
+"*. c #8587B3",
+"=. c #8688B5",
+"-. c #8E8FB0",
+";. c #8E8FB1",
+":. c #898BB7",
+">. c #8E90BB",
+",. c #959DB7",
+"<. c #9294BE",
+"1. c #9395BE",
+"2. c #9A9BB9",
+"3. c #9D9EBE",
+"4. c #89C382",
+"5. c #8FC688",
+"6. c #AEC4BA",
+"7. c #A5A5C1",
+"8. c #ABACC7",
+"9. c #AFB0CA",
+"0. c #B2B3CC",
+"q. c #B3B3CC",
+"w. c #B3B4CD",
+"e. c #B4B4CC",
+"r. c #B7B7CF",
+"t. c #C3C3D8",
+"y. c #C3C3D9",
+"u. c #CBCBDF",
+"i. c #CECEE1",
+"p. c #CFCFE2",
+"a. c #D0D0E2",
+"s. c #D2D2E3",
+"d. c #D3D3E4",
+"f. c #D4D4E5",
+"g. c None",
+"g.g.g.g.g.g.g.g.E 1.<.n   U 1.:.h   g.g.g.g.g.g.",
+"g.g.g.g.g.g.g.g. .1.1.I   R >.M   Z 2.g.g.g.g.g.",
+"g.g.g.g.g.g.g.g.l *.1.=.  | %.&.w.s.f.i.g.g.g.g.",
+"g.g.g.g.g.g.g.g.  f W 1.B O., 8 ..v d k g.g.g.g.",
+"g.g.g.g.g.g.g.g.c   O Y ,.- % % y G $.N   g.g.g.",
+"g.g.g.g.g.g.g.8.p.q.#.z @.1 % % p @ m 3.9.g.g.g.",
+"g.g.g.g.g.g.g.r.f.f.f.a.0.6.e u Q     . -.g.g.g.",
+"g.g.g.g.g.g.g.t.f.f.d.y.7.T C g         g.g.g.g.",
+"g.g.g.g.g.g.g.u.e.;.A o                 g.g.g.g.",
+"g.g.g.g.g.g.g.s       V ) ~ $         g.g.g.g.g.",
+"g.g.g.g.g.g.        H t = * o.L x   g.g.g.g.g.g.",
+"g.g.g.g.g.g.      D r % % 9 ( / j   g.g.g.g.g.g.",
+"g.g.g.g.g.g.    F i % % % ]           g.g.g.g.g.",
+"g.g.g.g.g.g.  J q % % % % (           g.g.g.g.g.",
+"g.g.g.g.g.  ` 4 % % % % > K             g.g.g.g.",
+"g.g.g.g.g.+.< % % % % % X.#             g.g.g.g.",
+"g.g.g.g.g.w % % % % & { S               g.g.g.g.",
+"g.g.g.g.g.> % % ; } _ b               g.g.g.g.g.",
+"g.g.g.g.6 % : [ P X                   g.g.g.g.g.",
+"g.g.g.7 % 2 ' +                       g.g.g.g.g.",
+"g.g.6 % 3 !                         g.g.g.g.g.g.",
+"g.0 % 5 ^                         g.g.g.g.g.g.g.",
+"a % 0 g.                      g.g.g.g.g.g.g.g.g.",
+"5.4.g.g.g.g.g.g.g.g.g.g.g.g.g.g.g.g.g.g.g.g.g.g."
+]
 
 
 spec = [
@@ -3961,11 +4172,13 @@ file_save = ["22 22 5 1",
 IconDict0 = {
     "derive": derive,
     "close": close,
+    "clipboard": clipboard,
     "fileclose": fileclose,
     "fileopen": file_open,
     "filesave": file_save,
     "fileprint": image_print_data,
     "spec": spec,
+    "bliss": bliss,
     "normal": normal,
     "normalize16": normalize16,
     "reload": reload_,
@@ -4031,6 +4244,7 @@ IconDict0 = {
 
 # corresponding silx icons
 TRANSLATION_TABLE = {
+    "clipboard": "edit-copy",
     "average16": "math-average",
     "derive": "math-derive",
     "close": "close",
@@ -4132,18 +4346,32 @@ class _PatchedIconDict(MutableMapping):
             self._qt = qt
             try:
                 from silx.gui import icons as silx_icons
+                IconDict = {}
             except ImportError:
                 _logger.debug("Could not import silx. Legacy icons will be used.")
                 silx_icons = None
+                from PyMca5.PyMcaGui.plotting.Silx_Icons import IconDict
 
             self._silx_icons = silx_icons
             # keep an internal copy:
             self._translation_table = TRANSLATION_TABLE.copy()
             self.__initialized = True
+            self._fallBackDict = IconDict
+
         if key not in self._unpatched_icons:
             raise KeyError("Unknown icon '%s'" % key)
 
-        if self._silx_icons is None or key not in TRANSLATION_TABLE:
+        if key not in TRANSLATION_TABLE:
+            _logger.debug("Using legacy icon '%s' because there is no "
+                          "corresponding icon.",
+                          key)
+            return self._unpatched_icons[key]
+
+        if self._silx_icons is None:
+            if TRANSLATION_TABLE[key] in self._fallBackDict:
+                _logger.info("Using fallback translation '%s' for '%s'" %
+                              (TRANSLATION_TABLE[key], key))
+                return self._fallBackDict[TRANSLATION_TABLE[key]]
             _logger.debug("Using legacy icon '%s' because silx is not "
                           "available or because it has no corresponding icon.",
                           key)
@@ -4177,19 +4405,34 @@ class _PatchedIconDict(MutableMapping):
             # we also need to remove the key from internal translation table
             del self._translation_table[key]
 
+
 IconDict = _PatchedIconDict(IconDict0)
 
 
+def change_icons(plot):
+    """Replace some of the silx icons with PyMca icons.
+
+    :param plot: Silx plot window, or SilxScanWindow, or SilxMcaWindow
+    :return:
+    """
+    from PyMca5.PyMcaGui import PyMcaQt as qt
+    plot.getRoiAction().setIcon(qt.QIcon(qt.QPixmap(IconDict["roi"])))
+    if hasattr(plot, "printPreview"):
+        plot.printPreview.setIcon(qt.QIcon(qt.QPixmap(IconDict["fileprint"])))
+
+
 def showIcons():
     w = qt.QWidget()
     g = qt.QGridLayout(w)
 
     idx = 0
-    for name, icon in IconDict.items():
-        print(name, type(icon))
-        column = int(idx / 10)
-        row = idx % 10
-        #print "name",name
+    keyList = list(IconDict.keys())
+    keyList.sort()
+    for key in keyList:
+        name = key
+        icon = IconDict[name]
+        column = int(idx / 20)
+        row = idx % 20
         lab = qt.QLabel(w)
         lab.setText(str(name))
         g.addWidget(lab, row, 2 * column + 1)
@@ -4209,3 +4452,4 @@ if __name__ == '__main__':
     _logger.setLevel(logging.DEBUG)
     w = showIcons()
     app.exec_()
+    app = None
