@@ -1,5 +1,5 @@
 #/*##########################################################################
-# Copyright (C) 2004-2018 V.A. Sole, European Synchrotron Radiation Facility
+# Copyright (C) 2004-2019 V.A. Sole, European Synchrotron Radiation Facility
 #
 # This file is part of the PyMca X-ray Fluorescence Toolkit developed at
 # the ESRF by the Software group.
@@ -27,12 +27,14 @@ __author__ = "V.A. Sole - ESRF Data Analysis"
 __contact__ = "sole@esrf.fr"
 __license__ = "MIT"
 __copyright__ = "European Synchrotron Radiation Facility, Grenoble, France"
+
 import sys
 import os
 import numpy
-#from numpy import argsort, nonzero, take
 import time
 import traceback
+import logging
+
 from PyMca5.PyMcaGui import PyMcaQt as qt
 if hasattr(qt, 'QString'):
     QString = qt.QString
@@ -45,12 +47,12 @@ import copy
 
 from PyMca5.PyMcaGui.io import PyMcaFileDialogs
 from PyMca5.PyMcaGui import IconDict
-from . import ScanWindow
+from .ScanWindow import ScanWindow
 from . import McaCalibrationControlGUI
 from PyMca5.PyMcaIO import ConfigDict
-from PyMca5.PyMcaGui import McaAdvancedFit
-from PyMca5.PyMcaCore import DataObject
+from PyMca5.PyMcaGui.physics.xrf import McaAdvancedFit
 from PyMca5.PyMcaGui.physics.xrf import McaCalWidget
+from PyMca5.PyMcaCore import DataObject
 from . import McaSimpleFit
 from PyMca5.PyMcaMath.fitting import Specfit
 from PyMca5.PyMcaMath.fitting import SpecfitFuns
@@ -60,19 +62,23 @@ from PyMca5 import PyMcaDirs
 from PyMca5.PyMcaGui import QPyMcaMatplotlibSave1D
 MATPLOTLIB = True
 
-#force understanding of utf-8 encoding
-#otherways it cannot generate svg output
+# force understanding of utf-8 encoding
+# otherwise it cannot generate svg output
 try:
     import encodings.utf_8
 except:
-    #not a big problem
+    # not a big problem
     pass
-DEBUG = 0
-class McaWindow(ScanWindow.ScanWindow):
+
+_logger = logging.getLogger(__name__)
+# _logger.setLevel(logging.DEBUG
+
+
+class McaWindow(ScanWindow):
     def __init__(self, parent=None, name="Mca Window", specfit=None, backend=None,
                  plugins=True, newplot=False, roi=True, fit=True, **kw):
 
-        ScanWindow.ScanWindow.__init__(self, parent,
+        ScanWindow.__init__(self, parent,
                                          name=name,
                                          newplot=newplot,
                                          plugins=plugins,
@@ -108,8 +114,7 @@ class McaWindow(ScanWindow.ScanWindow):
         self.advancedfit = McaAdvancedFit.McaAdvancedFit()
 
         self.printPreview = PyMcaPrintPreview.PyMcaPrintPreview(modal = 0)
-        if DEBUG:
-            print("printPreview id = %d" % id(self.printPreview))
+        _logger.debug("printPreview id = %d" % id(self.printPreview))
 
         self._buildCalibrationControlWidget()
         self._toggleCounter = 2
@@ -145,12 +150,12 @@ class McaWindow(ScanWindow.ScanWindow):
     def mcaSimpleFitSignal(self):
         legend = self.getActiveCurve(just_legend=True)
         if legend is None:
-           msg = qt.QMessageBox(self)
-           msg.setIcon(qt.QMessageBox.Critical)
-           msg.setText("Please Select an active curve")
-           msg.setWindowTitle('MCA Window Simple Fit')
-           msg.exec_()
-           return
+            msg = qt.QMessageBox(self)
+            msg.setIcon(qt.QMessageBox.Critical)
+            msg.setText("Please Select an active curve")
+            msg.setWindowTitle('MCA Window Simple Fit')
+            msg.exec_()
+            return
         x, y, info = self.getDataAndInfoFromLegend(legend)
         self.advancedfit.hide()
         self.simplefit.show()
@@ -161,16 +166,16 @@ class McaWindow(ScanWindow.ScanWindow):
             self.__simplefitcalmode = self.calibration
             curveinfo = info
             if self.calibration == 'None':
-                calib = [0.0,1.0,0.0]
+                calib = [0.0, 1.0, 0.0]
             else:
                 if 'McaCalib' in curveinfo:
                     calib = curveinfo['McaCalib']
                 else:
                     calib = [0.0, 1.0, 0.0]
             self.__simplefitcalibration = calib
-            calibrationOrder = curveinfo.get('McaCalibOrder',2)
+            calibrationOrder = curveinfo.get('McaCalibOrder', 2)
             if calibrationOrder == 'TOF':
-                x = calib[2] + calib[0] / pow(x-calib[1],2)
+                x = calib[2] + calib[0] / pow(x - alib[1],2)
             else:
                 x = calib[0] + calib[1] * x + calib[2] * x * x
             self.simplefit.setdata(x=x,y=y,
@@ -190,15 +195,14 @@ class McaWindow(ScanWindow.ScanWindow):
             if self.specfit.fitconfig['McaMode']:
                 self.simplefit.fit()
         else:
-                msg = qt.QMessageBox(self)
-                msg.setIcon(qt.QMessageBox.Critical)
-                msg.setText("Error. Trying to fit fitted data?")
-                msg.setWindowTitle('MCA Window Simple Fit')
-                msg.exec_()
+            msg = qt.QMessageBox(self)
+            msg.setIcon(qt.QMessageBox.Critical)
+            msg.setText("Error. Trying to fit fitted data?")
+            msg.setWindowTitle('MCA Window Simple Fit')
+            msg.exec_()
 
     def getActiveCurve(self, just_legend=False):
-        if DEBUG:
-            print("Local MCA window getActiveCurve called!!!!")
+        _logger.debug("Local MCA window getActiveCurve called!!!!")
         legend = super(McaWindow, self).getActiveCurve(just_legend)
         if just_legend:
             return legend
@@ -254,7 +258,7 @@ class McaWindow(ScanWindow.ScanWindow):
 
         if legend in self.dataObjectsDict:
             # The data as displayed
-            x, y, legend, curveinfo = self.getCurve(legend)
+            x, y, legend, curveinfo = self.getCurve(legend)[:4]
             # the data as first entered
             info  = self.dataObjectsDict[legend].info
             if self.calibration == 'None':
@@ -309,7 +313,7 @@ class McaWindow(ScanWindow.ScanWindow):
             elif 'McaCalibSource' in info:
                 calib = info['McaCalibSource']
             else:
-                calib = [0.0,1.0,0.0]
+                calib = [0.0, 1.0, 0.0]
         else:
             calib = curveinfo['McaCalib']
             energy = calib[0] + calib[1] * x + calib[2] * x * x
@@ -325,7 +329,7 @@ class McaWindow(ScanWindow.ScanWindow):
         self.advancedfit.raise_()
         if info is not None:
             xlabel = 'Channel'
-            self.advancedfit.setData(x=x,y=y,
+            self.advancedfit.setData(x=x, y=y,
                                      xmin=xmin,
                                      xmax=xmax,
                                      legend=legend,
@@ -342,9 +346,7 @@ class McaWindow(ScanWindow.ScanWindow):
         return
 
     def __anasignal(self,dict):
-        if DEBUG:
-            print("__anasignal called dict = ",dict)
-
+        _logger.debug("__anasignal called dict = ",dict)
         if dict['event'] == 'clicked':
             # A button has been cicked
             if   dict['button'] == 'Source':
@@ -593,8 +595,7 @@ class McaWindow(ScanWindow.ScanWindow):
                 """
                 pass
             else:
-                if DEBUG:
-                    print("Unknown combobox",dict['boxname'])
+                _logger.debug("Unknown combobox",dict['boxname'])
 
         elif (dict['event'] == 'EstimateFinished'):
             pass
@@ -694,8 +695,7 @@ class McaWindow(ScanWindow.ScanWindow):
                     else:
                         self.setGraphXLimits(emax, emin, replot=True)
             except:
-                if DEBUG:
-                    print("Refreshing due to exception", sys.exc_info()[1])
+                _logger.debug("Refreshing due to exception", sys.exc_info()[1])
                 self.refresh()
                 #self.graph.replot()
 
@@ -816,8 +816,7 @@ class McaWindow(ScanWindow.ScanWindow):
         elif dict['event'] == 'selectionChanged':
             print("Selection changed event not implemented any more")
         else:
-            if DEBUG:
-                print("Unknown or ignored event",dict['event'])
+            _logger.debug("Unknown or ignored event",dict['event'])
 
 
     def emitCurrentROISignal(self):
@@ -837,8 +836,7 @@ class McaWindow(ScanWindow.ScanWindow):
                 if legend in self.dataObjectsDict.keys():
                     A = self.dataObjectsDict[legend].x[0][0]
             except:
-                if DEBUG:
-                    print("X axis offset not found")
+                _logger.debug("X axis offset not found")
             B = 1.0
             C = 0.0
             order = 1
@@ -861,8 +859,7 @@ class McaWindow(ScanWindow.ScanWindow):
         w.sigReplaceSelection.connect(self._replaceSelection)
 
     def _addSelection(self, selection, replot=True):
-        if DEBUG:
-            print("__add, selection = ",selection)
+        _logger.debug("__add, selection = %s",selection)
 
         if type(selection) == type([]):
             sellist = selection
@@ -1106,8 +1103,7 @@ class McaWindow(ScanWindow.ScanWindow):
         self.updateLegends()
 
     def _removeSelection(self, selectionlist):
-        if DEBUG:
-            print("_removeSelection(self, selectionlist)",selectionlist)
+        _logger.debug("_removeSelection(self, selectionlist)",selectionlist)
         if type(selectionlist) == type([]):
             sellist = selectionlist
         else:
@@ -1142,8 +1138,7 @@ class McaWindow(ScanWindow.ScanWindow):
             self.replot()
 
     def _replaceSelection(self, selectionlist):
-        if DEBUG:
-            print("_replaceSelection(self, selectionlist)",selectionlist)
+        _logger.debug("_replaceSelection(self, selectionlist)",selectionlist)
         if type(selectionlist) == type([]):
             sellist = selectionlist
         else:
@@ -1164,8 +1159,7 @@ class McaWindow(ScanWindow.ScanWindow):
         self._addSelection(selectionlist)
 
     def graphCallback(self, ddict):
-        if DEBUG:
-            print("McaWindow._graphCallback", ddict)
+        _logger.debug("McaWindow._graphCallback", ddict)
         if ddict['event'] in ['markerMoved', 'markerSelected']:
             return self._handleMarkerEvent(ddict)
         elif ddict['event'] in ["mouseMoved", "MouseAt"]:
@@ -1207,7 +1201,7 @@ class McaWindow(ScanWindow.ScanWindow):
             return
         self.sigPlotSignal.emit(ddict)
 
-    def setActiveCurve(self, legend=None):
+    def setActiveCurve(self, legend=None, replot=True):
         if legend is None:
             legend = self.getActiveCurve(just_legend=True)
         if legend is None:
@@ -1257,7 +1251,8 @@ class McaWindow(ScanWindow.ScanWindow):
         super(McaWindow, self).setActiveCurve(legend, replot=False)
         self.setGraphXLabel(xlabel)
         self.setGraphYLabel(ylabel)
-        self.replot()
+        if replot:
+            self.replot()
 
     def _customFitSignalReceived(self, ddict):
         if ddict['event'] == "FitFinished":
@@ -1277,8 +1272,7 @@ class McaWindow(ScanWindow.ScanWindow):
                           own=True)
 
     def _scanFitSignalReceived(self, ddict):
-        if DEBUG:
-            print("_graphSignalReceived", ddict)
+        _logger.debug("_graphSignalReceived", ddict)
         if ddict['event'] == "EstimateFinished":
             return
         if ddict['event'] == "FitFinished":
@@ -1428,7 +1422,10 @@ class McaWindow(ScanWindow.ScanWindow):
         os.linesep = '\n'
         if filterused[0].upper() == "WIDGET":
             fformat = specFile[-3:].upper()
-            pixmap = qt.QPixmap.grabWidget(self.getWidgetHandle())
+            if hasattr(qt.QPixmap, "grabWidget"):
+                pixmap = qt.QPixmap.grabWidget(self.getWidgetHandle())
+            else:
+                pixmap = self.getWidgetHandle().grab()
             if not pixmap.save(specFile, fformat):
                 qt.QMessageBox.critical(self,
                         "Save Error",
@@ -1547,8 +1544,7 @@ class McaWindow(ScanWindow.ScanWindow):
 
     #The plugins interface
     def _toggleLogY(self):
-        if DEBUG:
-            print("McaWindow _toggleLogY")
+        _logger.debug("McaWindow _toggleLogY")
         self._ownSignal = True
         try:
             super(McaWindow, self)._toggleLogY()
@@ -1556,8 +1552,7 @@ class McaWindow(ScanWindow.ScanWindow):
             self._ownSignal = None
 
     def _toggleLogX(self):
-        if DEBUG:
-            print("McaWindow _toggleLogX")
+        _logger.debug("McaWindow _toggleLogX")
         self._ownSignal = True
         try:
             super(McaWindow, self)._toggleLogX()
@@ -1670,8 +1665,7 @@ class McaWindow(ScanWindow.ScanWindow):
             self._addSelection(sel_list, replot=replot)
 
     def refresh(self):
-        if DEBUG:
-            print(" DANGEROUS REFRESH CALLED")
+        _logger.debug(" DANGEROUS REFRESH CALLED")
         activeCurve = self.getActiveCurve(just_legend=True)
         # try to keep the same curve order
         legendList = self.getAllCurves(just_legend=True)
@@ -1724,7 +1718,7 @@ class McaWindow(ScanWindow.ScanWindow):
 def test():
     w = McaWindow()
     x = numpy.arange(1000.)
-    y =  10 * x + 10000. * numpy.exp(-0.5*(x-500)*(x-500)/400)
+    y = 10 * x + 10000. * numpy.exp(-0.5*(x-500)*(x-500)/400)
     w.addCurve(x, y, legend="dummy", replot=True, replace=True)
     w.resetZoom()
     app.lastWindowClosed.connect(app.quit)
@@ -1734,3 +1728,4 @@ def test():
 
 if __name__ == "__main__":
     test()
+    app = None
