@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 #/*##########################################################################
-# Copyright (C) 2004-2018 V.A. Sole, European Synchrotron Radiation Facility
+# Copyright (C) 2004-2019 V.A. Sole, European Synchrotron Radiation Facility
 #
 # This file is part of the PyMca X-ray Fluorescence Toolkit developed at
 # the ESRF by the Software group.
@@ -28,13 +28,16 @@ __author__ = "V.A. Sole - ESRF Data Analysis"
 __contact__ = "sole@esrf.fr"
 __license__ = "MIT"
 __copyright__ = "European Synchrotron Radiation Facility, Grenoble, France"
+import os
 import sys, getopt
 import traceback
+import logging
 if sys.platform == 'win32':
     import ctypes
     from ctypes.wintypes import MAX_PATH
+
 nativeFileDialogs = None
-DEBUG = 0
+_logger = logging.getLogger(__name__)
 backend=None
 if __name__ == '__main__':
     options     = '-f'
@@ -45,14 +48,15 @@ if __name__ == '__main__':
                    'backend=',
                    'nativefiledialogs=',
                    'PySide=',
-                   'binding=']
+                   'binding=',
+                   'logging=']
     try:
         opts, args = getopt.getopt(
                      sys.argv[1:],
                      options,
                      longoptions)
     except getopt.error:
-        print(sys.exc_info()[1])
+        print("%s" % sys.exc_info()[1])
         sys.exit(1)
 
     keywords={}
@@ -65,8 +69,10 @@ if __name__ == '__main__':
         elif opt in ('--shm'):
             keywords['shm']  = arg
         elif opt in ('--debug'):
-            debugreport = 1
-            DEBUG = 1
+            if arg.lower() not in ['0', 'false']:
+                debugreport = 1
+                _logger.setLevel(logging.DEBUG)
+            # --debug is also parsed later for the global logging level
         elif opt in ('-f'):
             keywords['fresh'] = 1
         elif opt in ('--qt'):
@@ -89,8 +95,13 @@ if __name__ == '__main__':
                 if sys.version_info < (3,):
                     try:
                         import sip
-                        sip.setapi("QString", 2)
-                        sip.setapi("QVariant", 2)
+                        sip.setapi('QString', 2)
+                        sip.setapi('QVariant', 2)
+                        sip.setapi('QDate', 2)
+                        sip.setapi('QDateTime', 2)
+                        sip.setapi('QTextStream', 2)
+                        sip.setapi('QTime', 2)
+                        sip.setapi('QUrl', 2)
                     except:
                         print("Cannot set sip API")
                 import PyQt4.QtCore
@@ -100,6 +111,15 @@ if __name__ == '__main__':
                 import PySide.QtCore
             else:
                 raise ValueError("Unknown Qt binding <%s>" % binding)
+
+    from PyMca5.PyMcaCore.LoggingLevel import getLoggingLevel
+    logging.basicConfig(level=getLoggingLevel(opts))
+    if "HDF5_USE_FILE_LOCKING" not in os.environ:
+        if "h5py" in sys.modules:
+             _logger.warning("h5py already imported")
+        os.environ["HDF5_USE_FILE_LOCKING"] = "FALSE"
+        _logger.info("%s set to %s" % ("HDF5_USE_FILE_LOCKING",
+                                       os.environ["HDF5_USE_FILE_LOCKING"]))
     if binding is None:
         if qtversion == '3':
             raise NotImplementedError("Qt3 is no longer supported")
@@ -107,14 +127,25 @@ if __name__ == '__main__':
             if sys.version_info < (3,):
                 try:
                     import sip
-                    sip.setapi("QString", 2)
-                    sip.setapi("QVariant", 2)
+                    sip.setapi('QString', 2)
+                    sip.setapi('QVariant', 2)
+                    sip.setapi('QDate', 2)
+                    sip.setapi('QDateTime', 2)
+                    sip.setapi('QTextStream', 2)
+                    sip.setapi('QTime', 2)
+                    sip.setapi('QUrl', 2)
                 except:
                     print("Cannot set sip API")
             import PyQt4.QtCore
         elif qtversion == '5':
             import PyQt5.QtCore
 
+    try:
+        # make sure hdf5plugins are imported
+        import hdf5plugin
+    except:
+        _logger.info("Failed to import hdf5plugin")
+
 from PyMca5.PyMcaGui import PyMcaQt as qt
 QTVERSION = qt.qVersion()
 if sys.platform == 'darwin':
@@ -123,7 +154,13 @@ if sys.platform == 'darwin':
             if hasattr(qt, 'QOpenGLWidget'):
                 print("Warning: OpenGL backend not fully supported")
 try:
-    # try to import silx prior to import matplotlib
+    import silx
+    if (sys.version_info < (3,)) and ("PyQt4.QtCore" in sys.modules):
+        # PyQt4 is the most reliable binding at the ESRF for python 2
+        silxLogger = logging.getLogger("silx.DEPRECATION")
+        silxLogger.setLevel(logging.CRITICAL)
+    # try to import silx prior to importing matplotlib to prevent
+    # unnecessary warning
     import silx.gui.plot
 except:
     pass
@@ -152,44 +189,24 @@ TOMOGUI_FLAG = True
 
 import PyMca5
 from PyMca5 import PyMcaDataDir
-import os
 __version__ = PyMca5.version()
 
 if __name__ == "__main__":
     sys.excepthook = qt.exceptionHandler
 
     app = qt.QApplication(sys.argv)
-    strlist = qt.QStyleFactory.keys()
-    if sys.platform == "win32":
-        for item in strlist:
-            text = str(item)
-            if text == "WindowsXP":
-                style = qt.QStyleFactory.create(item)
-                app.setStyle(style)
-                break
-
     if sys.platform not in ["win32", "darwin"]:
         # some themes of Ubuntu 16.04 give black tool tips on black background
         app.setStyleSheet("QToolTip { color: #000000; background-color: #fff0cd; border: 1px solid black; }")
 
-    # TODO why this strange test
-    if 1 or QTVERSION < '4.0.0':
-        winpalette = qt.QPalette(qt.QColor(230,240,249),qt.QColor(238,234,238))
-        app.setPalette(winpalette)
-    else:
-        palette = app.palette()
-        role = qt.QPalette.Window           #this is the background
-        palette.setColor(role, qt.QColor(238,234,238))
-        app.setPalette(palette)
-
     mpath = PyMcaDataDir.PYMCA_DATA_DIR
     if mpath[-3:] == "exe":
         mpath = os.path.dirname(mpath)
 
-    fname = os.path.join(mpath,'PyMcaSplashImage.png')
+    fname = os.path.join(mpath, 'PyMcaSplashImage.png')
     if not os.path.exists(fname):
        while len(mpath) > 3:
-         fname = os.path.join(mpath,'PyMcaSplashImage.png')
+         fname = os.path.join(mpath, 'PyMcaSplashImage.png')
          if not os.path.exists(fname):
              mpath = os.path.dirname(mpath)
          else:
@@ -205,9 +222,18 @@ if __name__ == "__main__":
     font = splash.font()
     font.setBold(1)
     splash.setFont(font)
-    splash.showMessage( 'PyMca %s' % __version__,
-            qt.Qt.AlignLeft|qt.Qt.AlignBottom,
-            qt.Qt.white)
+    try:
+        # there is a deprecation warning in Python 3.8 when
+        # dealing with the alignement flags.
+        alignment = int(qt.Qt.AlignLeft|qt.Qt.AlignBottom)
+        splash.showMessage( 'PyMca %s' % __version__,
+                alignment,
+                qt.Qt.white)
+    except:
+        # fall back to original implementation in case of troubles
+        splash.showMessage( 'PyMca %s' % __version__,
+                qt.Qt.AlignLeft|qt.Qt.AlignBottom,
+                qt.Qt.white)
     if sys.platform == "darwin":
         qApp = qt.QApplication.instance()
         qApp.processEvents()
@@ -218,19 +244,54 @@ from PyMca5.PyMcaGui.pymca import McaWindow
 
 from PyMca5.PyMcaGui.pymca import PyMcaImageWindow
 from PyMca5.PyMcaGui.pymca import PyMcaHKLImageWindow
+
 try:
     #This is to make sure it is properly frozen
     #and that Object3D is fully supported
     import OpenGL.GL
     #import Object3D.SceneGLWindow as SceneGLWindow
-    import PyMca5.PyMcaGui.pymca.PyMcaGLWindow as SceneGLWindow
+    from PyMca5.PyMcaGui.pymca import PyMcaGLWindow
     OBJECT3D = False
     if ("PyQt4.QtOpenGL" in sys.modules) or \
        ("PySide.QtOpenGL") in sys.modules or \
+       ("PySide2.QtOpenGL") in sys.modules or \
        ("PyQt5.QtOpenGL") in sys.modules:
         OBJECT3D = True
 except:
+    _logger.info("pyopengl not installed")
     OBJECT3D = False
+
+
+# Silx OpenGL availability (former Object3D)
+try:
+    import PyMca5.PyMcaGui.pymca.SilxGLWindow
+    isSilxGLAvailable = True
+except ImportError:
+    isSilxGLAvailable = False
+
+if isSilxGLAvailable:
+    SceneGLWindow = PyMca5.PyMcaGui.pymca.SilxGLWindow
+elif OBJECT3D:
+    SceneGLWindow = PyMcaGLWindow
+
+try:
+    from PyMca5.PyMcaGui.pymca import SilxScatterWindow
+except:
+    _logger.info("Cannot import SilxScatterWindow")
+
+_logger.info("SilxGL availability: %s", isSilxGLAvailable)
+
+# check that OpenGL is actually supported (ESRF rnice problems)
+OPENGL_DRIVERS_OK = True
+if sys.platform.startswith("linux"):
+    import subprocess
+    if subprocess.call("which glxinfo > /dev/null", shell=True) == 0:
+        if subprocess.call("glxinfo > /dev/null", shell=True):
+            OPENGL_DRIVERS_OK = False
+            isSilxGLAvailable = False
+            OBJECT3D = False
+            _logger.warning("OpenGL disabled. Errors using glxinfo command")
+
 from PyMca5.PyMcaGui.pymca import QDispatcher
 from PyMca5.PyMcaGui import ElementsInfo
 from PyMca5.PyMcaGui import PeakIdentifier
@@ -294,7 +355,7 @@ class PyMcaMain(PyMcaMdi.PyMcaMdi):
                          self.sourceWidget.sourceSelector._openFileSlot)
             self.openMenu.addAction("Load Training Data",
                                         self.loadTrainingData)
-
+            self.trainingDataMenu = None
 
             self.__useTabWidget = True
 
@@ -312,14 +373,16 @@ class PyMcaMain(PyMcaMdi.PyMcaMdi):
                 self.mainTabWidget = qt.QTabWidget(self.mdi)
                 self.mainTabWidget.setWindowTitle("Main Window")
                 self.mcaWindow = McaWindow.McaWindow(backend=backend)
+                self.mcaWindow.showRoiWidget(qt.Qt.BottomDockWidgetArea)
+
                 self.scanWindow = ScanWindow.ScanWindow(info=True,
                                                         backend=backend)
                 self.scanWindow._togglePointsSignal()
-                if OBJECT3D:
+                if OBJECT3D or isSilxGLAvailable:
                     self.glWindow = SceneGLWindow.SceneGLWindow()
                 self.mainTabWidget.addTab(self.mcaWindow, "MCA")
                 self.mainTabWidget.addTab(self.scanWindow, "SCAN")
-                if OBJECT3D:
+                if OBJECT3D or isSilxGLAvailable:
                     self.mainTabWidget.addTab(self.glWindow, "OpenGL")
                 if QTVERSION < '5.0.0':
                     self.mdi.addWindow(self.mainTabWidget)
@@ -380,7 +443,7 @@ class PyMcaMain(PyMcaMdi.PyMcaMdi):
                      self._startupSelection(source=kw['spec'],
                                                 selection=None)
 
-    def connectDispatcher(self, viewer, dispatcher = None):
+    def connectDispatcher(self, viewer, dispatcher=None):
         #I could connect sourceWidget to myself and then
         #pass the selections to the active window!!
         #That will be made in a next iteration I guess
@@ -411,6 +474,30 @@ class PyMcaMain(PyMcaMdi.PyMcaMdi):
                     return True
         return False
 
+    def _isScatterSelection(self, ddict):
+        if 'imageselection' in ddict:
+            if ddict['imageselection']:
+                return False
+        if 'selection' in ddict:
+            if ddict['selection'] is None:
+                return False
+            if "x" in ddict['selection']:
+                if hasattr(ddict['selection']['x'], "__len__"):
+                    if len(ddict['selection']['x']) != 2:
+                        return False
+                    size0 = ddict['dataobject'].x[0].size
+                    size1 = ddict['dataobject'].x[1].size
+                    if size0 != size1:
+                        return False
+                    if hasattr(ddict['dataobject'], "y"):
+                        if ddict['dataobject'].y:
+                            if ddict['dataobject'].y[0].size != size0 * size1:
+                                return False
+                    if "PyMca5.PyMcaGui.pymca.SilxScatterWindow" in sys.modules:
+                        if silx.version_info > (0, 10, 2):
+                            return True
+        return False
+
     def _is3DSelection(self, ddict):
         if self._is2DSelection(ddict):
             return False
@@ -423,7 +510,7 @@ class PyMcaMain(PyMcaMdi.PyMcaMdi):
                     return True
 
             if 'x' in ddict['selection']:
-                if ddict['selection']['x'] is not None:
+                if hasattr(ddict['selection']['x'], "__len__"):
                     if len(ddict['selection']['x']) > 1:
                         return True
         return False
@@ -447,31 +534,57 @@ class PyMcaMain(PyMcaMdi.PyMcaMdi):
             if self.mainTabWidget.isHidden():
                 #make sure it is visible in case of being closed
                 self.mainTabWidget.show()
-        if DEBUG:
+
+        try:
             self._dispatcherAddSelectionSlot(ddict)
-        else:
-            try:
-                self._dispatcherAddSelectionSlot(ddict)
-            except:
-                msg = qt.QMessageBox(self)
-                msg.setIcon(qt.QMessageBox.Critical)
-                msg.setText("Error: %s" % sys.exc_info()[1])
-                msg.setInformativeText(str(sys.exc_info()[1]))
-                msg.setDetailedText(traceback.format_exc())
-                msg.exec_()
+        except:
+            if _logger.getEffectiveLevel() == logging.DEBUG:
+                raise
+            msg = qt.QMessageBox(self)
+            msg.setIcon(qt.QMessageBox.Critical)
+            msg.setText("Error: %s" % sys.exc_info()[1])
+            msg.setInformativeText(str(sys.exc_info()[1]))
+            msg.setDetailedText(traceback.format_exc())
+            msg.exec_()
 
     def _dispatcherAddSelectionSlot(self, dictOrList):
-        if DEBUG:
-            print("self._dispatcherAddSelectionSlot(ddict), ddict = ", dictOrList)
+        _logger.info("self._dispatcherAddSelectionSlot(ddict), ddict = %s",
+                      dictOrList)
         if type(dictOrList) == type([]):
             ddict = dictOrList[0]
         else:
             ddict = dictOrList
 
         toadd = False
-        if self._is2DSelection(ddict):
-            if DEBUG:
-                print("2D selection")
+        if self._isScatterSelection(ddict):
+            _logger.info("ScatterPlot selection")
+            legend = ddict['legend']
+            if legend not in self.imageWindowDict.keys():
+                if OPENGL_DRIVERS_OK:
+                    backend = "gl"
+                else:
+                    backend = "mpl"
+                imageWindow = SilxScatterWindow.SilxScatterWindow(backend=backend)
+                self.imageWindowDict[legend] = imageWindow
+                self.imageWindowDict[legend].setPlotEnabled(True)
+                if self.mainTabWidget.indexOf(self.imageWindowDict[legend]) < 0:
+                    self.mainTabWidget.addTab(self.imageWindowDict[legend],
+                                          legend)
+                self.imageWindowDict[legend]._addSelection(ddict)
+                self.mainTabWidget.setCurrentWidget(imageWindow)
+                return
+            if self.mainTabWidget.indexOf(self.imageWindowDict[legend]) < 0:
+                self.mainTabWidget.addTab(self.imageWindowDict[legend],
+                                          legend)
+                #self.imageWindowDict[legend].setPlotEnabled(False)
+                self.imageWindowDict[legend]._addSelection(ddict)
+                self.mainTabWidget.setCurrentWidget(self.imageWindowDict\
+                                                        [legend])
+            else:
+                self.imageWindowDict[legend]._addSelection(ddict)
+            return
+        elif self._is2DSelection(ddict):
+            _logger.info("2D selection")
             if self.imageWindowCorrelator is None:
                 self.imageWindowCorrelator = RGBCorrelator.RGBCorrelator()
                 #toadd = True
@@ -493,13 +606,15 @@ class PyMcaMain(PyMcaMdi.PyMcaMdi):
                 except:
                     pass
                 if hkl:
-                    imageWindow = PyMcaHKLImageWindow.PyMcaHKLImageWindow(name = legend,
-                                correlator = self.imageWindowCorrelator,
-                                scanwindow=self.scanWindow)
+                    imageWindow = PyMcaHKLImageWindow.PyMcaHKLImageWindow(
+                            name=legend,
+                            correlator=self.imageWindowCorrelator,
+                            scanwindow=self.scanWindow)
                 else:
-                    imageWindow = PyMcaImageWindow.PyMcaImageWindow(name = legend,
-                                correlator = self.imageWindowCorrelator,
-                                scanwindow=self.scanWindow)
+                    imageWindow = PyMcaImageWindow.PyMcaImageWindow(
+                            name=legend,
+                            correlator=self.imageWindowCorrelator,
+                            scanwindow=self.scanWindow)
                 self.imageWindowDict[legend] = imageWindow
 
                 imageWindow.sigAddImageClicked.connect( \
@@ -527,8 +642,7 @@ class PyMcaMain(PyMcaMdi.PyMcaMdi):
             else:
                 self.imageWindowDict[legend]._addSelection(ddict)
         elif self._isStackSelection(ddict):
-            if DEBUG:
-                print("Stack selection")
+            _logger.info("Stack selection")
             legend = ddict['legend']
             widget = QStackWidget.QStackWidget()
             widget.notifyCloseEventToWidget(self)
@@ -537,20 +651,17 @@ class PyMcaMain(PyMcaMdi.PyMcaMdi):
             widget.show()
             self._widgetDict[id(widget)] = widget
         else:
-            if OBJECT3D:
+            if OBJECT3D or isSilxGLAvailable:
                 if ddict['dataobject'].info['selectiontype'] == "1D":
-                    if DEBUG:
-                        print("1D selection")
+                    _logger.info("1D selection")
                     self.mcaWindow._addSelection(dictOrList)
                     self.scanWindow._addSelection(dictOrList)
                 else:
-                    if DEBUG:
-                        print("3D selection")
+                    _logger.info("3D selection")
                     self.mainTabWidget.setCurrentWidget(self.glWindow)
                     self.glWindow._addSelection(dictOrList)
             else:
-                if DEBUG:
-                    print("1D selection")
+                _logger.info("1D selection")
                 self.mcaWindow._addSelection(dictOrList)
                 self.scanWindow._addSelection(dictOrList)
 
@@ -565,13 +676,12 @@ class PyMcaMain(PyMcaMdi.PyMcaMdi):
 
 
     def _dispatcherRemoveSelectionSlot(self, dictOrList):
-        if DEBUG:
-            print("self.dispatcherRemoveSelectionSlot(ddict), ddict = ", dictOrList)
+        _logger.debug("self.dispatcherRemoveSelectionSlot(ddict), ddict = %s", dictOrList)
         if type(dictOrList) == type([]):
             ddict = dictOrList[0]
         else:
             ddict = dictOrList
-        if self._is2DSelection(ddict):
+        if self._isScatterSelection(ddict) or self._is2DSelection(ddict):
             legend = ddict['legend']
             if legend in self.imageWindowDict.keys():
                 index = self.mainTabWidget.indexOf(self.imageWindowDict[legend])
@@ -597,13 +707,13 @@ class PyMcaMain(PyMcaMdi.PyMcaMdi):
             msg.exec_()
 
     def _dispatcherReplaceSelectionSlot(self, dictOrList):
-        if DEBUG:
-            print("self.dispatcherReplaceSelectionSlot(ddict), ddict = ", dictOrList)
+        _logger.debug("self.dispatcherReplaceSelectionSlot(ddict), ddict = %s",
+                      dictOrList)
         if type(dictOrList) == type([]):
             ddict = dictOrList[0]
         else:
             ddict = dictOrList
-        if self._is2DSelection(ddict):
+        if self._isScatterSelection(ddict) or self._is2DSelection(ddict):
             legend = ddict['legend']
             for key in list(self.imageWindowDict.keys()):
                 index = self.mainTabWidget.indexOf(self.imageWindowDict[key])
@@ -632,8 +742,8 @@ class PyMcaMain(PyMcaMdi.PyMcaMdi):
             self.scanWindow._replaceSelection(dictOrList)
 
     def dispatcherOtherSignalsSlot(self, dictOrList):
-        if DEBUG:
-            print("self.dispatcherOtherSignalsSlot(ddict), ddict = ",dictOrList)
+        _logger.debug("self.dispatcherOtherSignalsSlot(ddict), ddict = %s",
+                      dictOrList)
         if type(dictOrList) == type([]):
             ddict = dictOrList[0]
         else:
@@ -652,8 +762,7 @@ class PyMcaMain(PyMcaMdi.PyMcaMdi):
         if ddict['event'] == "SourceTypeChanged":
             pass
             return
-        if DEBUG:
-            print("Unhandled dict")
+        _logger.debug("Unhandled dict")
 
     def setConfig(self, configDict):
         if 'PyMca' in configDict:
@@ -729,7 +838,7 @@ class PyMcaMain(PyMcaMdi.PyMcaMdi):
         d["PyMca"]["McaWindow"]["calibrations"] = self.mcaWindow.getCalibrations()
 
         #ROIs
-        d['ROI']={}
+        d['ROI'] = {}
         if self.mcaWindow.roiWidget is None:
             roilist = []
             roidict = {}
@@ -763,15 +872,14 @@ class PyMcaMain(PyMcaMdi.PyMcaMdi):
         #ScanFit related
         d['ScanSimpleFit'] = {}
         d['ScanSimpleFit']['Configuration'] = {}
-        if DEBUG:
-                  d['ScanSimpleFit']['Configuration'].update(\
-                      self.scanWindow.scanFit.getConfiguration())
-        else:
-            try:
-                  d['ScanSimpleFit']['Configuration'].update(\
-                      self.scanWindow.scanFit.getConfiguration())
-            except:
-                print("Error getting ScanFint configuration")
+
+        try:
+            d['ScanSimpleFit']['Configuration'].update(
+                self.scanWindow.scanFit.getConfiguration())
+        except:
+            if _logger.getEffectiveLevel() == logging.DEBUG:
+                raise
+            _logger.warning("Error getting ScanFit configuration")
         return d
 
     def saveConfig(self, config, filename = None):
@@ -792,17 +900,17 @@ class PyMcaMain(PyMcaMdi.PyMcaMdi):
             key = 'Splitter'
             if key in ddict['Geometry'].keys():
                 self.splitter.setSizes(ddict['Geometry'][key])
+            key = 'McaWindow'
+            if key in ddict['Geometry'].keys():
+                r = qt.QRect(*ddict['Geometry']['McaWindow'])
+                self.mcaWindow.setGeometry(r)
             if hasattr(self.mcaWindow, "graph"):
                 # this was the way of working of 4.x.x versions
-                key = 'McaWindow'
-                if key in ddict['Geometry'].keys():
-                    r = qt.QRect(*ddict['Geometry']['McaWindow'])
-                    self.mcaWindow.setGeometry(r)
                 key = 'McaGraph'
                 if key in ddict['Geometry'].keys():
                     r = qt.QRect(*ddict['Geometry']['McaGraph'])
                     self.mcaWindow.graph.setGeometry(r)
-                self.show()
+            self.show()
             qApp = qt.QApplication.instance()
             qApp.processEvents()
             qApp.postEvent(self, qt.QResizeEvent(qt.QSize(ddict['Geometry']['MainWindow'][2]+1,
@@ -941,7 +1049,7 @@ class PyMcaMain(PyMcaMdi.PyMcaMdi):
                         except:
                             pass
                 else:
-                    print("hidden")
+                    _logger.info("hidden")
 
     def __configureScanCustomFit(self, ddict):
         pass
@@ -957,13 +1065,14 @@ class PyMcaMain(PyMcaMdi.PyMcaMdi):
             self.actionOpen = qt.QAction(self)
             self.actionOpen.setText(QString("&Open"))
             self.actionOpen.setIcon(self.Icons["fileopen"])
-            self.actionOpen.setShortcut(qt.Qt.CTRL+qt.Qt.Key_O)
+            self.actionOpen.setShortcut(qt.QKeySequence(qt.Qt.CTRL+qt.Qt.Key_O))
             self.actionOpen.triggered[bool].connect(self.onOpen)
             #filesaveas
             self.actionSaveAs = qt.QAction(self)
             self.actionSaveAs.setText(QString("&Save"))
             self.actionSaveAs.setIcon(self.Icons["filesave"])
-            self.actionSaveAs.setShortcut(qt.Qt.CTRL+qt.Qt.Key_S)
+            self.actionSaveAs.setShortcut(\
+                qt.QKeySequence(qt.Qt.CTRL+qt.Qt.Key_S))
             self.actionSaveAs.triggered[bool].connect(self.onSaveAs)
 
             #filesave
@@ -977,14 +1086,16 @@ class PyMcaMain(PyMcaMdi.PyMcaMdi):
             self.actionPrint = qt.QAction(self)
             self.actionPrint.setText(QString("&Print"))
             self.actionPrint.setIcon(self.Icons["fileprint"])
-            self.actionPrint.setShortcut(qt.Qt.CTRL+qt.Qt.Key_P)
+            self.actionPrint.setShortcut(\
+                qt.QKeySequence(qt.Qt.CTRL+qt.Qt.Key_P))
             self.actionPrint.triggered[bool].connect(self.onPrint)
 
             #filequit
             self.actionQuit = qt.QAction(self)
             self.actionQuit.setText(QString("&Quit"))
             #self.actionQuit.setIcon(self.Icons["fileprint"])
-            self.actionQuit.setShortcut(qt.Qt.CTRL+qt.Qt.Key_Q)
+            self.actionQuit.setShortcut(\
+                                       qt.QKeySequence(qt.Qt.CTRL+qt.Qt.Key_Q))
             qApp = qt.QApplication.instance()
             self.actionQuit.triggered.connect(qApp.closeAllWindows)
 
@@ -1115,8 +1226,7 @@ class PyMcaMain(PyMcaMdi.PyMcaMdi):
         """
 
     def menuToolsAboutToShow(self):
-        if DEBUG:
-            print("menu ToolsAboutToShow")
+        _logger.debug("menu ToolsAboutToShow")
         self.menuTools.clear()
         if self.sourceFrame.isHidden():
             self.menuTools.addAction("Show Source",self.toggleSource)
@@ -1139,8 +1249,7 @@ class PyMcaMain(PyMcaMdi.PyMcaMdi):
                                      self._xrfmcPyMca)
         if SUMRULES_FLAG:
             self.menuTools.addAction("Sum Rules Tool", self._sumRules)
-        if DEBUG:
-            print("Fit to Specfile missing")
+        _logger.debug("Fit to Specfile missing")
         if TOMOGUI_FLAG:
             self.menuTools.addAction("Tomography reconstruction",
                                      self.__tomoRecons)
@@ -1153,8 +1262,7 @@ class PyMcaMain(PyMcaMdi.PyMcaMdi):
 
 
     def toggleSource(self,**kw):
-        if DEBUG:
-            print("toggleSource called")
+        _logger.debug("toggleSource called")
         if self.sourceFrame.isHidden():
             self.sourceFrame.show()
             self.sourceFrame.raise_()
@@ -1226,11 +1334,7 @@ class PyMcaMain(PyMcaMdi.PyMcaMdi):
             correlator.raise_()
         self.__correlator[-1].sigRGBCorrelatorSignal.connect( \
                 self._deleteCorrelator)
-
-        if len(filelist) == 1:
-            correlator.addBatchDatFile(filelist[0])
-        else:
-            correlator.addFileList(filelist)
+        correlator.addFileList(filelist)
 
     def __sumRules(self):
         if self.__correlator is None:
@@ -1395,8 +1499,9 @@ class PyMcaMain(PyMcaMdi.PyMcaMdi):
                         if widget.parent() is None:
                             widget.close()
                 except:
-                    if DEBUG:
-                        print("Error closing widget")
+                    _logger.debug("Error closing widget")
+            from PyMca5.PyMcaGui.plotting import PyMcaPrintPreview
+            PyMcaPrintPreview.resetSingletonPrintPreview()
         return PyMcaMdi.PyMcaMdi.closeEvent(self, event)
 
     def __xiaCorrect(self):
@@ -1505,22 +1610,43 @@ class PyMcaMain(PyMcaMdi.PyMcaMdi):
         self.saveConfig(self.getConfig(), filename)
 
     def loadTrainingData(self):
+        if self.trainingDataMenu is None:
+            self.trainingDataMenu = qt.QMenu()
+            self.trainingSources = {"XRF Analysis": os.path.join(PyMcaDataDir.PYMCA_DATA_DIR, 'XRFSpectrum.mca'),
+                        "Tertiary Excitation": os.path.join(PyMcaDataDir.PYMCA_DATA_DIR, 'Steel.spe')}
+            self.trainingActions = []
+            for key in ["XRF Analysis", "Tertiary Excitation"]:
+                action = qt.QAction(key, None)
+                self.trainingActions.append(action)
+                self.trainingDataMenu.addAction(action)
         try:
-            source = os.path.join(PyMcaDataDir.PYMCA_DATA_DIR,
-                                    'XRFSpectrum.mca')
-            self.sourceWidget.sourceSelector.openSource(source)
+            source = None
+            selectedAction = self.trainingDataMenu.exec_(qt.QCursor.pos())
+            if selectedAction is not None:
+                key = qt.safe_str(selectedAction.text())
+                source = self.trainingSources[key]
+                self.sourceWidget.sourceSelector.openSource(source)
+                # only in case of the steel sample we set the input dir to simplify accessing the supplied cfg file
+                if key in ["Tertiary Excitation"]:
+                    # we do not change the input dir currently used by the source selector
+                    #self.sourceWidget.sourceSelector.lastInputDir = os.path.dirname(source)
+                    # but we change the input dir to allow easy loading of the config file from the
+                    # fit configuration window
+                    PyMcaDirs.inputDir = os.path.dirname(source)
         except:
             msg = qt.QMessageBox(self)
             msg.setIcon(qt.QMessageBox.Critical)
             msg.setWindowTitle("Error opening data source")
-            msg.setText("Cannot open data source %s" % source)
+            if source:
+                msg.setText("Cannot open data source %s" % source)
+            else:
+                msg.setText("Cannot open data source")
             msg.setInformativeText(str(sys.exc_info()[1]))
             msg.setDetailedText(traceback.format_exc())
             msg.exec_()
 
     def openSource(self,index=0):
-        if DEBUG:
-            print("index = %d " % index)
+        _logger.debug("index = %d ", index)
         if index <= 0:
             outfile = qt.QFileDialog(self)
             outfile.setWindowTitle("Select PyMca Configuration File")
@@ -1635,18 +1761,21 @@ class PyMcaMain(PyMcaMdi.PyMcaMdi):
         self.changeLog.show()
 
     def onDebug(self):
-        print("Module name PyQt  ",qt.PYQT_VERSION_STR)
+        if "PyQt5.QtCore" in sys.modules:
+            _logger.debug("Module name PyQt  %s", qt.PYQT_VERSION_STR)
         for module in sys.modules.values():
             try:
                 if 'Revision' in module.__revision__:
                     if module.__name__ != "__main__":
-                        print("Module name = ",module.__name__,module.__revision__.replace("$",""))
+                        _logger.debug("Module name = %s, %s",
+                                      module.__name__,
+                                      module.__revision__.replace("$", ""))
             except:
                 pass
 
     def onPrint(self):
-        if DEBUG:
-            print("onPrint called")
+        _logger.debug("onPrint called")
+
         if not self.scanWindow.isHidden():
             self.scanWindow.printGraph()
             return
@@ -1671,7 +1800,7 @@ if 0:
 
             #name= self.__getNewGraphName()
             if name == "MCA Graph":
-                graph= McaWindow.McaWindow(self.mdi, name=name)
+                graph = McaWindow.McaWindow(self.mdi, name=name)
             graph.windowClosed[()].connect(self.closeGraph)
             graph.show()
 
@@ -1704,7 +1833,7 @@ if 0:
             """
             Called after a graph is closed
             """
-            print("closeGraph", name)
+            _logger.info("closeGraph", name)
 
     def __getGraphNames(self):
             return [ str(window.caption()) for window in self.mdi.windowList() ]
@@ -1744,8 +1873,7 @@ class MyQTextBrowser(qt.QTextBrowser):
 class Line(qt.QFrame):
     sigLineDoubleClickEvent = qt.pyqtSignal(object)
     def mouseDoubleClickEvent(self,event):
-        if DEBUG:
-            print("Double Click Event")
+        _logger.debug("Double Click Event")
         ddict={}
         ddict['event']="DoubleClick"
         ddict['data'] = event
@@ -1754,8 +1882,7 @@ class Line(qt.QFrame):
 class PixmapLabel(qt.QLabel):
     sigPixmapLabelMousePressEvent = qt.pyqtSignal(object)
     def mousePressEvent(self,event):
-        if DEBUG:
-            print("Mouse Press Event")
+        _logger.debug("Mouse Press Event")
         ddict={}
         ddict['event']="MousePress"
         ddict['data'] = event
@@ -1796,5 +1923,7 @@ if __name__ == '__main__':
         p=pstats.Stats("test")
         p.strip_dirs().sort_stats(-1).print_stats()
     else:
-        sys.exit(app.exec_())
+        ret = app.exec_()
+        app = None
+        sys.exit(ret)
 
