@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 #/*##########################################################################
-# Copyright (C) 2004-2018 V.A. Sole, European Synchrotron Radiation Facility
+# Copyright (C) 2004-2019 V.A. Sole, European Synchrotron Radiation Facility
 #
 # This file is part of the PyMca X-ray Fluorescence Toolkit developed at
 # the ESRF by the Software group.
@@ -33,22 +33,40 @@ import os
 import traceback
 import numpy
 import weakref
+import logging
+_logger = logging.getLogger(__name__)
 
 if __name__ == "__main__":
+    if "HDF5_USE_FILE_LOCKING" not in os.environ:
+        if "h5py" in sys.modules:
+             _logger.warning("h5py already imported")
+        os.environ["HDF5_USE_FILE_LOCKING"] = "FALSE"
+        _logger.info("%s set to %s" % ("HDF5_USE_FILE_LOCKING",
+                                       os.environ["HDF5_USE_FILE_LOCKING"]))
+    try:
+        # make sure hdf5plugins are imported
+        import hdf5plugin
+    except:
+        _logger.info("Failed to import hdf5plugin")
     # we have to get the Qt binding prior to import PyMcaQt
     import getopt
     options = ''
     longoptions = ["fileindex=","old",
                    "filepattern=", "begin=", "end=", "increment=",
                    "nativefiledialogs=", "imagestack=", "image=",
-                   "backend=", "binding="]
+                   "backend=", "binding=", "logging=", "debug="]
     opts, args = getopt.getopt(
                  sys.argv[1:],
                  options,
                  longoptions)
     binding = None
     for opt, arg in opts:
-        if opt in ('--binding'):
+        if opt in ('--debug'):
+            if arg.lower() not in ['0', 'false']:
+                debugreport = 1
+                _logger.setLevel(logging.DEBUG)
+            # --debug is also parsed later for the global logging level
+        elif opt in ('--binding'):
             binding = arg.lower()
             if binding == "pyqt5":
                 import PyQt5.QtCore
@@ -56,8 +74,13 @@ if __name__ == "__main__":
                 if sys.version_info < (3,):
                     try:
                         import sip
-                        sip.setapi("QString", 2)
-                        sip.setapi("QVariant", 2)
+                        sip.setapi('QString', 2)
+                        sip.setapi('QVariant', 2)
+                        sip.setapi('QDate', 2)
+                        sip.setapi('QDateTime', 2)
+                        sip.setapi('QTextStream', 2)
+                        sip.setapi('QTime', 2)
+                        sip.setapi('QUrl', 2)
                     except:
                         print("Cannot set sip API")
                 import PyQt4.QtCore
@@ -67,14 +90,22 @@ if __name__ == "__main__":
                 import PySide.QtCore
             else:
                 raise ValueError("Unknown Qt binding <%s>" % binding)
+    from PyMca5.PyMcaCore.LoggingLevel import getLoggingLevel
+    logging.basicConfig(level=getLoggingLevel(opts))
+
 from PyMca5.PyMcaGui import PyMcaQt as qt
 if hasattr(qt, "QString"):
     QString = qt.QString
 else:
     QString = qt.safe_str
 try:
-    # try to import silx prior to importing matplotlib
-    # to prevent unnecessary warning
+    if (sys.version_info < (3,)) and ("PyQt4.QtCore" in sys.modules):
+        # PyQt4 is the most reliable binding at the ESRF for python 2
+        import silx
+        silxLogger = logging.getLogger("silx.DEPRECATION")
+        silxLogger.setLevel(logging.CRITICAL)
+    # try to import silx prior to importing matplotlib to prevent
+    # unnecessary warning
     import silx.gui.plot
 except:
     pass
@@ -87,7 +118,6 @@ from PyMca5.PyMcaGui import CloseEventNotifyingWidget
 from PyMca5.PyMcaGui import MaskImageWidget
 convertToRowAndColumn = MaskImageWidget.convertToRowAndColumn
 
-from PyMca5.PyMcaGui.pymca import StackROIWindow
 from PyMca5.PyMcaGui.pymca import RGBCorrelator
 from PyMca5.PyMcaGui.pymca.RGBCorrelatorWidget import ImageShapeDialog
 from PyMca5.PyMcaGui import IconDict
@@ -96,19 +126,19 @@ from PyMca5 import PyMcaDirs
 from PyMca5.PyMcaIO import ArraySave
 HDF5 = ArraySave.HDF5
 
-DEBUG = 0
+# _logger.setLevel(logging.DEBUG)
 QTVERSION = qt.qVersion()
-if DEBUG:
-    StackBase.DEBUG = DEBUG
+if _logger.getEffectiveLevel() == logging.DEBUG:
+    StackBase.logger.setLevel(logging.DEBUG)
 
 
 class QStackWidget(StackBase.StackBase,
                    CloseEventNotifyingWidget.CloseEventNotifyingWidget):
-    def __init__(self, parent = None,
-                 mcawidget = None,
-                 rgbwidget = None,
-                 vertical = False,
-                 master = True):
+    def __init__(self, parent=None,
+                 mcawidget=None,
+                 rgbwidget=None,
+                 vertical=False,
+                 master=True):
         StackBase.StackBase.__init__(self)
         CloseEventNotifyingWidget.CloseEventNotifyingWidget.__init__(self,
                                                                      parent)
@@ -202,16 +232,16 @@ class QStackWidget(StackBase.StackBase,
                                                          standalonesave=standaloneSaving,
                                                          profileselection=True,
                                                          aspect=True)
-        infotext  = 'Toggle background subtraction from current image\n'
+        infotext = 'Toggle background subtraction from current image\n'
         infotext += 'subtracting a straight line between the ROI limits.'
         self.roiBackgroundIcon = qt.QIcon(qt.QPixmap(IconDict["subtract"]))
         self.roiBackgroundButton = self.roiWidget.graphWidget._addToolButton(\
                                     self.roiBackgroundIcon,
                                     self._roiSubtractBackgroundClicked,
                                     infotext,
-                                    toggle = True,
-                                    state = False,
-                                    position = 6)
+                                    toggle=True,
+                                    state=False,
+                                    position=6)
         self.roiGraphWidget = self.roiWidget.graphWidget
         self.stackWindow.mainLayout.addWidget(self.stackWidget)
         self.roiWindow.mainLayout.addWidget(self.roiWidget)
@@ -223,38 +253,38 @@ class QStackWidget(StackBase.StackBase,
 
 
         #add some missing icons
-        offset = 6
-        infotext  = 'If checked, spectra will be added normalized to the number\n'
+        offset = 8
+        infotext = 'If checked, spectra will be added normalized to the number\n'
         infotext += 'of pixels. Be carefull if you are preparing a batch and you\n'
         infotext += 'fit the normalized spectra because the data in the batch will\n'
         infotext += 'have a different weight because they are not normalized.'
         self.normalizeIcon = qt.QIcon(qt.QPixmap(IconDict["normalize16"]))
-        self.normalizeButton = self.stackGraphWidget._addToolButton(\
+        self.normalizeButton = self.stackGraphWidget._addToolButton( \
                                         self.normalizeIcon,
                                         self.normalizeIconChecked,
                                         infotext,
-                                        toggle = True,
-                                        state = False,
-                                        position = 6)
+                                        toggle=True,
+                                        state=False,
+                                        position=6)
         offset += 1
 
         if self.master:
             self.loadIcon = qt.QIcon(qt.QPixmap(IconDict["fileopen"]))
-            self.loadStackButton = self.stackGraphWidget._addToolButton(\
+            self.loadStackButton = self.stackGraphWidget._addToolButton( \
                                         self.loadIcon,
                                         self.loadSlaveStack,
                                         'Load another stack of same shape',
-                                        position = offset)
+                                        position=offset)
             offset += 1
 
-        self.pluginIcon     = qt.QIcon(qt.QPixmap(IconDict["plugin"]))
+        self.pluginIcon = qt.QIcon(qt.QPixmap(IconDict["plugin"]))
         infotext = "Call/Load Stack Plugins"
         self.stackGraphWidget._addToolButton(self.pluginIcon,
                                              self._pluginClicked,
                                              infotext,
-                                             toggle = False,
-                                             state = False,
-                                             position = offset)
+                                             toggle=False,
+                                             state=False,
+                                             position=offset)
 
     def setStack(self, *var, **kw):
         self.stackWidget.setImageData(None)
@@ -263,7 +293,7 @@ class QStackWidget(StackBase.StackBase,
         if (1 in self._stack.data.shape) and\
            isinstance(self._stack.data, numpy.ndarray):
             oldshape = self._stack.data.shape
-            dialog = ImageShapeDialog(self, shape = oldshape[0:2])
+            dialog = ImageShapeDialog(self, shape=oldshape[0:2])
             dialog.setModal(True)
             ret = dialog.exec_()
             if ret:
@@ -304,14 +334,14 @@ class QStackWidget(StackBase.StackBase,
         xScale = self._stack.info.get("xScale", None)
         yScale = self._stack.info.get("yScale", None)
         if self.roiBackgroundButton.isChecked():
-            self.roiWidget.graphWidget.graph.setGraphTitle(\
-                                self._ROIImageNames[0]+" Net")
-            self.roiWidget.setImageData(self._ROIImageList[0]-\
+            self.roiWidget.graphWidget.graph.setGraphTitle( \
+                                self._ROIImageNames[0] + " Net")
+            self.roiWidget.setImageData(self._ROIImageList[0] - \
                                         self._ROIImageList[-1],
                                         xScale=xScale,
                                         yScale=yScale)
         else:
-            self.roiWidget.graphWidget.graph.setGraphTitle(\
+            self.roiWidget.graphWidget.graph.setGraphTitle( \
                                 self._ROIImageNames[0])
             self.roiWidget.setImageData(self._ROIImageList[0],
                                         xScale=xScale,
@@ -398,7 +428,7 @@ class QStackWidget(StackBase.StackBase,
 
         ArraySave.save3DArrayAsMonochromaticTiff(dataView,
                                     filename,
-                                    labels = None,
+                                    labels=None,
                                     dtype=dtype,
                                     mcaindex=mcaIndex)
 
@@ -442,7 +472,7 @@ class QStackWidget(StackBase.StackBase,
             row1 = int(min([row1+0.5, self._stack.data.shape[0]]))
             col0 = int(max([col0+0.5, 0]))
             col1 = int(min([col1+0.5, self._stack.data.shape[1]]))
-            view = self._stack.data[row0:row1+1, col0:col1+1,:]
+            view = self._stack.data[row0:row1+1, col0:col1+1, :]
         return view
 
     def saveStackAsNeXus(self, dtype=None, interpretation=None, compression=False):
@@ -528,14 +558,14 @@ class QStackWidget(StackBase.StackBase,
         if not len(filename):
             return
         ArraySave.save3DArrayAsHDF5(self._stack.data, filename,
-                                    labels = None, dtype=None, mode='nexus+')
+                                    labels=None, dtype=None, mode='nexus+')
 
     def saveStackAsSimpleHDF5(self):
         filename = self._getOutputHDF5Filename()
         if not len(filename):
             return
         ArraySave.save3DArrayAsHDF5(self._stack.data, filename,
-                                    labels = None, dtype=None, mode='simple')
+                                    labels=None, dtype=None, mode='simple')
 
     def saveStackAsSimplestHDF5(self):
         filename = self._getOutputHDF5Filename()
@@ -543,15 +573,15 @@ class QStackWidget(StackBase.StackBase,
             return
         view = self._getCroppedView()
         ArraySave.save3DArrayAsHDF5(view, filename,
-                                    labels = None, dtype=None, mode='simplest')
+                                    labels=None, dtype=None, mode='simplest')
 
     def loadStack(self):
         if self._stackImageData is not None:
             #clear with a small stack
             stack = DataObject.DataObject()
-            stack.data = numpy.zeros((100,100,100), numpy.float32)
+            stack.data = numpy.zeros((100, 100, 100), numpy.float32)
             self.setStack(stack)
-        if self.stackSelector  is None:
+        if self.stackSelector is None:
             self.stackSelector = StackSelector.StackSelector(self)
         stack = self.stackSelector.getStack()
         if (type(stack) == type([])) or isinstance(stack, list):
@@ -635,7 +665,7 @@ class QStackWidget(StackBase.StackBase,
             else:
                 self._closeSlave()
                 return
-        if self.stackSelector  is None:
+        if self.stackSelector is None:
             self.stackSelector = StackSelector.StackSelector(self)
 
         try:
@@ -725,8 +755,8 @@ class QStackWidget(StackBase.StackBase,
         text = QString("Set User Plugin Directory")
         menu.addAction(text)
         actionList.append(text)
-        global DEBUG
-        if DEBUG:
+        global _logger
+        if _logger.getEffectiveLevel() == logging.DEBUG:
             text = QString("Toggle DEBUG mode OFF")
         else:
             text = QString("Toggle DEBUG mode ON")
@@ -785,11 +815,12 @@ class QStackWidget(StackBase.StackBase,
                 self.setPluginDirectoryList(pluginsDirList)
             return
         if idx == 2:
-            if DEBUG:
-                DEBUG = 0
+            if _logger.getEffectiveLevel() == logging.DEBUG:
+                _logger.setLevel(logging.DEBUG)
+                StackBase.logger.setLevel(logging.DEBUG)
             else:
-                DEBUG = 1
-            StackBase.DEBUG = DEBUG
+                _logger.setLevel(logging.NOTSET)
+                StackBase.logger.setLevel(logging.NOTSET)
             return
         key = callableKeys[idx]
         methods = self.pluginInstanceDict[key].getMethods()
@@ -832,7 +863,7 @@ class QStackWidget(StackBase.StackBase,
             msg.setInformativeText(qt.safe_str(sys.exc_info()[1]))
             msg.setDetailedText(traceback.format_exc())
             msg.exec_()
-            if DEBUG:
+            if _logger.getEffectiveLevel() == logging.DEBUG:
                 raise
 
     def _actionHovered(self, action):
@@ -858,10 +889,11 @@ class QStackWidget(StackBase.StackBase,
                 self.mainLayout.addWidget(self.rgbWidget)
         elif n == 2:
             self.tab = qt.QTabWidget(self)
-            self.mcaWidget = McaWindow.McaWindow()#vertical=False)
+            self.mcaWidget = McaWindow.McaWindow() #vertical=False
             #self.mcaWidget.graph.setMinimumWidth(0.5 * \
             #                            qt.QWidget.sizeHint(self).width())
-            self.tab.setMaximumHeight(1.3 * qt.QWidget.sizeHint(self).height())
+            self.tab.setMaximumHeight(int(1.3 * \
+                                          qt.QWidget.sizeHint(self).height()))
             self.mcaWidget.setWindowTitle("PyMCA - Mca Window")
             self.tab.addTab(self.mcaWidget, "MCA")
             self.rgbWidget = RGBCorrelator.RGBCorrelator()
@@ -909,10 +941,11 @@ class QStackWidget(StackBase.StackBase,
         # self.stackGraphWidget.setInfoText("    X = ???? Y = ???? Z = ????")
         # self.stackGraphWidget.showInfo()
 
-        self.stackGraphWidget.graph.sigPlotSignal.connect(\
+        self.stackGraphWidget.graph.sigPlotSignal.connect( \
                                     self._stackGraphSignal)
+
         self.mcaWidget.sigROISignal.connect(self._mcaWidgetSignal)
-        self.roiWidget.graphWidget.graph.sigPlotSignal.connect(\
+        self.roiWidget.graphWidget.graph.sigPlotSignal.connect( \
                                     self._stackGraphSignal)
 
     def showOriginalImage(self):
@@ -923,8 +956,8 @@ class QStackWidget(StackBase.StackBase,
         xScale = self._stack.info.get("xScale", None)
         yScale = self._stack.info.get("yScale", None)
         self.stackWidget.setImageData(self._stackImageData,
-                                        xScale=xScale,
-                                        yScale=yScale)
+                                      xScale=xScale,
+                                      yScale=yScale)
 
     def showOriginalMca(self):
         goodData = numpy.isfinite(self._mcaData0.y[0].sum())
@@ -936,7 +969,7 @@ class QStackWidget(StackBase.StackBase,
         msg = qt.QMessageBox(self)
         msg.setIcon(qt.QMessageBox.Information)
         msg.setWindowTitle("Non finite data")
-        text  = "Your data contain infinite values or nans.\n"
+        text = "Your data contain infinite values or nans.\n"
         text += "Pixels containing those values will be ignored."
         msg.setText(text)
         msg.exec_()
@@ -950,7 +983,7 @@ class QStackWidget(StackBase.StackBase,
             msg = qt.QMessageBox(self)
             msg.setIcon(qt.QMessageBox.Information)
             msg.setWindowTitle("No active curve selected")
-            text  = "Please select the MCA active curve."
+            text = "Please select the MCA active curve."
             msg.setText(text)
             msg.exec_()
             return
@@ -965,8 +998,8 @@ class QStackWidget(StackBase.StackBase,
         xScale = self._stack.info.get("xScale", None)
         yScale = self._stack.info.get("yScale", None)
         if self.roiBackgroundButton.isChecked():
-            self.roiWidget.graphWidget.graph.setGraphTitle(image_names[0]+\
-                                                      " Net")
+            self.roiWidget.graphWidget.graph.setGraphTitle(image_names[0] + \
+                                                           " Net")
             self.roiWidget.setImageData(imageList[0]-imageList[-1],
                                         xScale=xScale,
                                         yScale=yScale)
@@ -1037,18 +1070,18 @@ class QStackWidget(StackBase.StackBase,
 
         if self.normalizeButton.isChecked():
             if self._selectionMask is None:
-                npixels = self._stackImageData.shape[0] *\
+                npixels = self._stackImageData.shape[0] * \
                           self._stackImageData.shape[1]
             else:
                 npixels = self._selectionMask.sum()
                 if npixels == 0:
-                    npixels = self._stackImageData.shape[0] *\
+                    npixels = self._stackImageData.shape[0] * \
                               self._stackImageData.shape[1]
             legend += "/%d" % npixels
         return self.sendMcaSelection(dataObject,
-                          key = "Selection",
-                          legend =legend,
-                          action = action)
+                                     key="Selection",
+                                     legend=legend,
+                                     action=action)
 
     def _removeMcaClicked(self):
         #remove the mca
@@ -1063,7 +1096,7 @@ class QStackWidget(StackBase.StackBase,
                 if curve.startswith(legend):
                     legend = curve
                     break
-        self.sendMcaSelection(dataObject, legend = legend, action = "REMOVE")
+        self.sendMcaSelection(dataObject, legend=legend, action="REMOVE")
 
     def _replaceMcaClicked(self):
         #replace the mca
@@ -1071,7 +1104,7 @@ class QStackWidget(StackBase.StackBase,
         self._addMcaClicked(action="REPLACE")
         self.__ROIConnected = True
 
-    def sendMcaSelection(self, mcaObject, key = None, legend = None, action = None):
+    def sendMcaSelection(self, mcaObject, key=None, legend=None, action=None):
         if action is None:
             action = "ADD"
         if key is None:
@@ -1084,9 +1117,9 @@ class QStackWidget(StackBase.StackBase,
                 legend += "/%d" % npixels
         sel = {}
         sel['SourceName'] = "EDF Stack"
-        sel['Key']        =  key
-        sel['legend']     =  legend
-        sel['dataobject'] =  mcaObject
+        sel['Key'] = key
+        sel['legend'] = legend
+        sel['dataobject'] = mcaObject
         if action == "ADD":
             self.mcaWidget._addSelection([sel])
         elif action == "REMOVE":
@@ -1106,11 +1139,10 @@ class QStackWidget(StackBase.StackBase,
         if instance_id == id(self):
             return
 
-        if DEBUG:
-            if self._slaveList is not None:
-                print("MASTER  setSelectionMask CALLED")
-            elif self._masterStack is not None:
-                print("SLAVE setSelectionMask CALLED")
+        if self._slaveList is not None:
+            _logger.debug("MASTER  setSelectionMask CALLED")
+        elif self._masterStack is not None:
+            _logger.debug("SLAVE setSelectionMask CALLED")
 
         #inform built in widgets
         for widget in [self.stackWidget, self.roiWidget]:
@@ -1131,8 +1163,7 @@ class QStackWidget(StackBase.StackBase,
                     instanceList.append(id(slave.pluginInstanceDict[key]))
                 if instance_id not in instanceList:
                     #Originated by the master
-                    if DEBUG:
-                        print("INFORMING SLAVE")
+                    _logger.warning("INFORMING SLAVE")
                     slave.setSelectionMask(mask, instance_id=id(self))
 
         if self._masterStack is not None:
@@ -1143,8 +1174,7 @@ class QStackWidget(StackBase.StackBase,
                 instanceList.append(id(self.pluginInstanceDict[key]))
             if instance_id in instanceList:
                 #Originated by the slave
-                if DEBUG:
-                    print("INFORMING MASTER")
+                _logger.debug("INFORMING MASTER")
                 self._masterStack.setSelectionMask(mask, instance_id=id(self))
 
         #Inform plugins
@@ -1198,8 +1228,8 @@ class QStackWidget(StackBase.StackBase,
             x = min(int(x), limits[0]-1)
             y = min(int(y), limits[1]-1)
             z = self._stackImageData[x, y]
-            self.stackGraphWidget.setInfoText("    X = %d Y = %d Z = %.4g" %\
-                                               (y, x, z))
+            self.stackGraphWidget.setInfoText( \
+                    "    X = %d Y = %d Z = %.4g" % (y, x, z))
 
     def _mcaWidgetSignal(self, ddict):
         if not self.__ROIConnected:
@@ -1237,6 +1267,19 @@ class QStackWidget(StackBase.StackBase,
         for key in self.pluginInstanceDict.keys():
             self.pluginInstanceDict[key].stackClosed()
         CloseEventNotifyingWidget.CloseEventNotifyingWidget.closeEvent(self, event)
+        if __name__ == "__main__":
+            app = qt.QApplication.instance()
+            allWidgets = app.allWidgets()
+            for widget in allWidgets:
+                try:
+                    # we cannot afford to crash here
+                    if id(widget) != id(self):
+                        if widget.parent() is None:
+                            widget.close()
+                except:
+                    _logger.debug("Error closing widget")
+            from PyMca5.PyMcaGui.plotting import PyMcaPrintPreview
+            PyMcaPrintPreview.resetSingletonPrintPreview()
 
 
 def test():
@@ -1248,7 +1291,7 @@ def test():
     stackData = numpy.zeros((nrows, ncols, nchannels), numpy.float)
     for i in range(nchannels):
         stackData[:, :, i] = a * i
-    stackData[0:10,:,:] = 0
+    stackData[0:10, :, :] = 0
     w = QStackWidget()
     w.setStack(stackData, mcaindex=2)
     w.show()
@@ -1263,7 +1306,7 @@ if __name__ == "__main__":
                      options,
                      longoptions)
     except:
-        print(sys.exc_info()[1])
+        print("%s" % sys.exc_info()[1])
         sys.exit(1)
     fileindex = 0
     filepattern=None
@@ -1273,6 +1316,7 @@ if __name__ == "__main__":
     increment=None
     backend=None
     PyMcaDirs.nativeFileDialogs=True
+
     for opt, arg in opts:
         if opt in '--begin':
             if "," in arg:
@@ -1290,17 +1334,17 @@ if __name__ == "__main__":
             else:
                 increment = int(arg)
         elif opt in '--filepattern':
-            filepattern = arg.replace('"','')
-            filepattern = filepattern.replace("'","")
+            filepattern = arg.replace('"', '')
+            filepattern = filepattern.replace("'", "")
         elif opt in '--fileindex':
             fileindex = int(arg)
         elif opt in ['--imagestack', "--image"]:
             imagestack = int(arg)
         elif opt in '--nativefiledialogs':
             if int(arg):
-                PyMcaDirs.nativeFileDialogs=True
+                PyMcaDirs.nativeFileDialogs = True
             else:
-                PyMcaDirs.nativeFileDialogs=False
+                PyMcaDirs.nativeFileDialogs = False
         elif opt in '--backend':
             backend = arg
         #elif opt in '--old':
@@ -1319,7 +1363,7 @@ if __name__ == "__main__":
             from PyMca5.PyMcaGraph.Plot import Plot
             Plot.defaultBackend = backend
         except:
-            print("WARNING: Cannot set backend to %s" % backend)
+            _logger.warning("WARNING: Cannot set backend to %s", backend)
     widget = QStackWidget()
     w = StackSelector.StackSelector(widget)
     if filepattern is not None:
@@ -1343,4 +1387,5 @@ if __name__ == "__main__":
         widget.setStack(stack)
     widget.show()
     app.exec_()
+    app = None
 
