@@ -2,7 +2,7 @@
 #
 # The PyMca X-Ray Fluorescence Toolkit
 #
-# Copyright (c) 2004-2018 European Synchrotron Radiation Facility
+# Copyright (c) 2004-2019 European Synchrotron Radiation Facility
 #
 # This file is part of the PyMca X-ray Fluorescence Toolkit developed at
 # the ESRF by the Software group.
@@ -75,7 +75,7 @@ __copyright__ = "European Synchrotron Radiation Facility, Grenoble, France"
     - There are some fields in the header that are required for this implementation. If any of
       these is missing, or inconsistent, it will be generated an error:
         Size: Represents size of data block
-        Dim_1: size of x coordinates (Dim_2 for 2-dimentional images, and also Dim_3 for 3d)
+        Dim_1: size of x coordinates (Dim_2 for 2-dimensional images, and also Dim_3 for 3d)
         DataType
         ByteOrder
     - For the written images, these fields are automatically genereted:
@@ -92,10 +92,10 @@ __copyright__ = "European Synchrotron Radiation Facility, Grenoble, France"
       section is going to be translated into an 1D, 2D or 3D Numpy Array, and accessed
       through GetData method call.
 """
-DEBUG = 0
 ################################################################################
 import sys
 import numpy
+import logging
 import os.path #, tempfile, shutil
 try:
     import gzip
@@ -130,6 +130,9 @@ try:
 except:
     CAN_USE_FASTEDF = 0
 
+_logger = logging.getLogger(__name__)
+
+
 ################################################################################
 # constants
 HEADER_BLOCK_SIZE = 1024
@@ -630,7 +633,7 @@ class  EdfFile(object):
                 try:
                     datasize = self.__GetSizeNumpyType__(datatype)
                 except TypeError:
-                    print("What is the meaning of this error?")
+                    _logger.debug("What is the meaning of this error?")
                     datasize = 8
                 if self.Images[Index].NumDim == 3:
                     sizeToRead = self.Images[Index].Dim1 * \
@@ -702,7 +705,7 @@ class  EdfFile(object):
 
         else:
             if fastedf:
-                print("I could not use fast routines")
+                _logger.info("I could not use fast routines")
             type_ = self.__GetDefaultNumpyType__(self.Images[Index].DataType, index=Index)
             size_pixel = self.__GetSizeNumpyType__(type_)
             Data = numpy.array([], type_)
@@ -902,7 +905,7 @@ class  EdfFile(object):
                                      self.__GetSizeNumpyType__(Data.dtype))
             self.Images[Index].NumDim = 3
         elif len(Data.shape) > 3:
-            raise TypeError("EdfFile: Data dimension not suported")
+            raise TypeError("EdfFile: Data dimension not supported")
 
 
         if DataType == "":
@@ -953,20 +956,16 @@ class  EdfFile(object):
     #Internal Methods
 
     def __makeSureFileIsOpen(self):
-        if DEBUG:
-            print("Making sure file is open")
+        _logger.debug("Making sure file is open")
         if not self.__ownedOpen:
             return
         if self.ADSC or self.MARCCD or self.PILATUS_CBF or self.SPE:
-            if DEBUG:
-                print("Special case. Image is buffered")
+            _logger.debug("Special case. Image is buffered")
             return
         if self.File in [0, None]:
-            if DEBUG:
-                print("File is None")
+            _logger.debug("File is None")
         elif self.File.closed:
-            if DEBUG:
-                print("Reopening closed file")
+            _logger.debug("Reopening closed file")
             accessMode = self.File.mode
             fileName = self.File.name
             newFile = open(fileName, accessMode)
@@ -974,20 +973,16 @@ class  EdfFile(object):
         return
 
     def __makeSureFileIsClosed(self):
-        if DEBUG:
-            print("Making sure file is closed")
+        _logger.debug("Making sure file is closed")
         if not self.__ownedOpen:
             return
         if self.ADSC or self.MARCCD or self.PILATUS_CBF or self.SPE:
-            if DEBUG:
-                print("Special case. Image is buffered")
+            _logger.debug("Special case. Image is buffered")
             return
         if self.File in [0, None]:
-            if DEBUG:
-                print("File is None")
+            _logger.debug("File is None")
         elif not self.File.closed:
-            if DEBUG:
-                print("Closing file")
+            _logger.debug("Closing file")
             self.File.close()
         return
 
