@@ -2,7 +2,7 @@
 #
 # The PyMca X-Ray Fluorescence Toolkit
 #
-# Copyright (c) 2004-2018 European Synchrotron Radiation Facility
+# Copyright (c) 2004-2019 European Synchrotron Radiation Facility
 #
 # This file is part of the PyMca X-ray Fluorescence Toolkit developed at
 # the ESRF by the Software group.
@@ -33,22 +33,14 @@ __copyright__ = "European Synchrotron Radiation Facility, Grenoble, France"
 import posixpath
 import numpy
 import h5py
-try:
-    from PyMca5.PyMcaCore import DataObject
-    from PyMca5.PyMcaMisc import PhysicalMemory
-except ImportError:
-    print("HDF5Stack1D importing DataObject from local directory!")
-    import DataObject
-    import PhysicalMemory
-try:
-    from PyMca5.PyMcaCore import NexusDataSource
-    from PyMca5.PyMcaCore import NexusTools
-except ImportError:
-    print("HDF5Stack1D importing NexusDataSource from local directory!")
-    import NexusDataSource
-    import NexusDataSource
+import logging
+_logger = logging.getLogger(__name__)
+
+from PyMca5.PyMcaCore import DataObject
+from PyMca5.PyMcaMisc import PhysicalMemory
+from PyMca5.PyMcaCore import NexusDataSource
+from PyMca5.PyMcaCore import NexusTools
 
-DEBUG = 0
 SOURCE_TYPE = "HDF5Stack1D"
 
 class HDF5Stack1D(DataObject.DataObject):
@@ -80,12 +72,11 @@ class HDF5Stack1D(DataObject.DataObject):
                  Example: The actual path has the form:
                  /whatever1/whatever2/counts
                  That means scanlist = ["/whatever1"]
-                 and               selection['y'] = "/whatever2/counts"
+                 and selection['y'] = "/whatever2/counts"
         """
-        if DEBUG:
-            print("filelist = ", filelist)
-            print("selection = ", selection)
-            print("scanlist = ", scanlist)
+        _logger.info("filelist = %s", filelist)
+        _logger.info("selection = %s", selection)
+        _logger.info("scanlist = %s", scanlist)
         # all the files in the same source
         hdfStack = NexusDataSource.NexusDataSource(filelist)
 
@@ -94,8 +85,11 @@ class HDF5Stack1D(DataObject.DataObject):
         tmpHdf = hdfStack._sourceObjectList[0]
         entryNames = []
         for key in tmpHdf["/"].keys():
-            if isinstance(tmpHdf["/"+key], h5py.Group):
-                entryNames.append(key)
+            try:
+                if isinstance(tmpHdf["/"+key], h5py.Group):
+                    entryNames.append(key)
+            except KeyError:
+                _logger.info("Broken link with key? <%s>" % key)
 
         # built the selection in terms of HDF terms
         # for the time being, only the first item in x selection used
@@ -186,7 +180,7 @@ class HDF5Stack1D(DataObject.DataObject):
                                     # this is the case of a selection inside a group
                                     scanlist.append("1.%d" % i)
                         except KeyError:
-                            print("%s not in file, ignoring." % dirname)
+                            _logger.warning("%s not in file, ignoring.", dirname)
                     if not len(scanlist):
                         if not ySelection.startswith("/"):
                             path = "/" + ySelection
@@ -231,9 +225,8 @@ class HDF5Stack1D(DataObject.DataObject):
             if not nScans:
                 raise IOError("No entry contains the required data")
 
-        if DEBUG:
-            print("Retained number of files = %d" % nFiles)
-            print("Retained number of scans = %d" % nScans)
+        _logger.debug("Retained number of files = %d", nFiles)
+        _logger.debug("Retained number of scans = %d", nScans)
 
         # Now is to decide the number of mca ...
         # I assume all the scans contain the same number of mca
@@ -264,8 +257,7 @@ class HDF5Stack1D(DataObject.DataObject):
                                       numpy.float64]:
                 # Some datasets form CLS (origin APS?) arrive as data format
                 # equal to ">u2" and are not triggered as integer types
-                if DEBUG:
-                    print("Not basic dataset type %s" % self.__dtype)
+                _logger.debug("Not basic dataset type %s", self.__dtype)
                 if ("%s" % self.__dtype).endswith("2"):
                     self.__dtype = numpy.float32
                 else:
@@ -279,8 +271,7 @@ class HDF5Stack1D(DataObject.DataObject):
         mcaIndex = selection.get('index', len(shape)-1)
         if mcaIndex == -1:
             mcaIndex = len(shape) - 1
-        if DEBUG:
-            print("mcaIndex = %d" % mcaIndex)
+        _logger.debug("mcaIndex = %d", mcaIndex)
         considerAsImages = False
         dim0, dim1, mcaDim = self.getDimensions(nFiles, nScans, shape,
                                                 index=mcaIndex)
@@ -295,17 +286,24 @@ class HDF5Stack1D(DataObject.DataObject):
                 bytefactor = 8
 
             neededMegaBytes = nFiles * dim0 * dim1 * (mcaDim * bytefactor/(1024*1024.))
+            _logger.info("Using %d bytes per item" % bytefactor)
+            _logger.info("Needed %d Megabytes" % neededMegaBytes)
             physicalMemory = None
             if hasattr(PhysicalMemory, "getAvailablePhysicalMemoryOrNone"):
                 physicalMemory = PhysicalMemory.getAvailablePhysicalMemoryOrNone()
             if not physicalMemory:
                 physicalMemory = PhysicalMemory.getPhysicalMemoryOrNone()
+            else:
+                _logger.info("Available physical memory %.1f GBytes" % \
+                             (physicalMemory/(1024*1024*1024.)))
             if physicalMemory is None:
                 # 6 Gigabytes of available memory
                 # should be a good compromise in 2018
                 physicalMemory = 6000
+                _logger.info("Assumed physical memory %.1f MBytes" % physicalMemory)
             else:
                 physicalMemory /= (1024*1024.)
+            _logger.info("Using physical memory %.1f GBytes" % (physicalMemory/1024))
             if (neededMegaBytes > (0.95*physicalMemory))\
                and (nFiles == 1) and (len(shape) == 3):
                 if self.__dtype0 is None:
@@ -328,9 +326,9 @@ class HDF5Stack1D(DataObject.DataObject):
         except (MemoryError, ValueError):
             # some versions report ValueError instead of MemoryError
             if (nFiles == 1) and (len(shape) == 3):
-                print("Attempting dynamic loading")
+                _logger.warning("Attempting dynamic loading")
                 if mSelection is not None:
-                    print("Ignoring monitor")
+                    _logger.warning("Ignoring monitor")
                 self.data = yDataset
                 if mSelection is not None:
                     mdtype = tmpHdf[mpath].dtype
@@ -339,7 +337,7 @@ class HDF5Stack1D(DataObject.DataObject):
                     mDataset = numpy.asarray(tmpHdf[mpath], dtype=mdtype)
                     self.monitor = [mDataset]
                 if xSelection is not None:
-                    xDataset = tmpHdf[xpath].value
+                    xDataset = tmpHdf[xpath][()]
                     self.x = [xDataset]
                 if h5py.version.version < '2.0':
                     #prevent automatic closing keeping a reference
@@ -350,6 +348,16 @@ class HDF5Stack1D(DataObject.DataObject):
                 # what to do if the number of dimensions is only 2?
                 raise
 
+        # get the positioners information associated to the path
+        positioners = {}
+        try:
+            positionersGroup = NexusTools.getPositionersGroup(tmpHdf, path)
+            for motorName, motorValues in positionersGroup.items():
+                positioners[motorName] = motorValues[()]
+        except:
+            positionersGroup = None
+            positioners = {}
+
         # get the mca information associated to the path
         mcaObjectPaths = NexusTools.getMcaObjectPaths(tmpHdf, path)
         _time = None
@@ -364,62 +372,63 @@ class HDF5Stack1D(DataObject.DataObject):
                     if DONE:
                         # hopefully it will fit into memory
                         if mcaObjectPaths["live_time"] in tmpHdf:
-                            _time = tmpHdf[mcaObjectPaths["live_time"]].value
+                            _time = tmpHdf[mcaObjectPaths["live_time"]][()]
                         elif "::" in mcaObjectPaths["live_time"]:
                             tmpFileName, tmpDatasetPath = \
-                                        mcaObjectPaths["live_time"].split("::") 
+                                        mcaObjectPaths["live_time"].split("::")
                             with h5py.File(tmpFileName, "r") as tmpH5:
-                                _time = tmpH5[tmpDatasetPath].value
+                                _time = tmpH5[tmpDatasetPath][()]
                         else:
                             del mcaObjectPaths["live_time"]
                     else:
                         # we have to have as many live times as MCA spectra
                         _time = numpy.zeros( \
                                     (self.data.shape[0] * self.data.shape[1]),
-                                    dtype=numpy.float64)                
+                                    dtype=numpy.float64)
                 elif "elapsed_time" in mcaObjectPaths:
                     if DONE:
                         # hopefully it will fit into memory
                         if mcaObjectPaths["elapsed_time"] in tmpHdf:
                             _time = \
-                                tmpHdf[mcaObjectPaths["elapsed_time"]].value
+                                tmpHdf[mcaObjectPaths["elapsed_time"]][()]
                         elif "::" in mcaObjectPaths["elapsed_time"]:
                             tmpFileName, tmpDatasetPath = \
-                                    mcaObjectPaths["elapsed_time"].split("::") 
+                                    mcaObjectPaths["elapsed_time"].split("::")
                             with h5py.File(tmpFileName, "r") as tmpH5:
-                                _time = tmpH5[tmpDatasetPath].value
+                                _time = tmpH5[tmpDatasetPath][()]
                         else:
                             del mcaObjectPaths["elapsed_time"]
                     else:
                         # we have to have as many elpased times as MCA spectra
                         _time = numpy.zeros((self.data.shape[0] * self.data.shape[1]),
-                                                numpy.float32)                    
+                                                numpy.float32)
                 if "calibration" in mcaObjectPaths:
                     if mcaObjectPaths["calibration"] in tmpHdf:
                         _calibration = \
-                                tmpHdf[mcaObjectPaths["calibration"]].value
+                                tmpHdf[mcaObjectPaths["calibration"]][()]
                     elif "::" in mcaObjectPaths["calibration"]:
                         tmpFileName, tmpDatasetPath = \
-                                    mcaObjectPaths["calibration"].split("::") 
+                                    mcaObjectPaths["calibration"].split("::")
                         with h5py.File(tmpFileName, "r") as tmpH5:
-                            _calibration = tmpH5[tmpDatasetPath].value
+                            _calibration = tmpH5[tmpDatasetPath][()]
                     else:
                         del mcaObjectPaths["calibration"]
                 if "channels" in mcaObjectPaths:
                     if mcaObjectPaths["channels"] in tmpHdf:
                         _channels = \
-                                tmpHdf[mcaObjectPaths["channels"]].value
+                                tmpHdf[mcaObjectPaths["channels"]][()]
                     elif "::" in mcaObjectPaths["channels"]:
                         tmpFileName, tmpDatasetPath = \
-                                    mcaObjectPaths["channels"].split("::") 
+                                    mcaObjectPaths["channels"].split("::")
                         with h5py.File(tmpFileName, "r") as tmpH5:
-                            _channels = tmpH5[tmpDatasetPath].value
+                            _channels = tmpH5[tmpDatasetPath][()]
                     else:
                         del mcaObjectPaths["channels"]
             else:
                 self._pathHasRelevantInfo = False
 
         if (not DONE) and (not considerAsImages):
+            _logger.info("Data in memory as spectra")
             self.info["McaIndex"] = 2
             n = 0
 
@@ -433,17 +442,17 @@ class HDF5Stack1D(DataObject.DataObject):
                 goodEntryNames = []
                 for entry in entryNames:
                     tmpPath = "/" + entry
-                    if hasattr(hdf[tmpPath], "keys"):
-                        goodEntryNames.append(entry)
+                    try:
+                        if hasattr(hdf[tmpPath], "keys"):
+                            goodEntryNames.append(entry)
+                    except KeyError:
+                        _logger.info("Broken link with key? <%s>" % tmpPath)
+
                 for scan in scanlist:
                     IN_MEMORY = None
                     nStart = n
                     for ySelection in ySelectionList:
                         n = nStart
-                        if IN_MEMORY == False:
-                            # We can only deal with one dynamic dataset
-                            print("Selection %s ignored" % ySelection)
-                            continue
                         if JUST_KEYS:
                             entryName = goodEntryNames[int(scan.split(".")[-1])-1]
                             path = entryName + ySelection
@@ -455,7 +464,7 @@ class HDF5Stack1D(DataObject.DataObject):
                                 mDataset = numpy.asarray(hdf[mpath], dtype=mdtype)
                             if xSelection is not None:
                                 xpath = entryName + xSelection
-                                xDataset = hdf[xpath].value
+                                xDataset = hdf[xpath][()]
                         else:
                             path = scan + ySelection
                             if mSelection is not None:
@@ -466,21 +475,25 @@ class HDF5Stack1D(DataObject.DataObject):
                                 mDataset = numpy.asarray(hdf[mpath], dtype=mdtype)
                             if xSelection is not None:
                                 xpath = scan + xSelection
-                                xDataset = hdf[xpath].value
+                                xDataset = hdf[xpath][()]
                         try:
                             yDataset = hdf[path]
                             tmpShape = yDataset.shape
                             totalBytes = numpy.ones((1,), yDataset.dtype).itemsize
                             for nItems in tmpShape:
                                 totalBytes *= nItems
-                            if (totalBytes/(1024.*1024.)) > 500:
+                            # should one be conservative or just try?
+                            if (totalBytes/(1024.*1024.)) > (0.4 * physicalMemory):
+                                _logger.info("Force dynamic loading of spectra")
                                 #read from disk
                                 IN_MEMORY = False
                             else:
                                 #read the data into memory
-                                yDataset = hdf[path].value
+                                _logger.info("Attempt to load whole map into memory")
+                                yDataset = hdf[path][()]
                                 IN_MEMORY = True
                         except (MemoryError, ValueError):
+                            _logger.info("Dynamic loading of spectra")
                             yDataset = hdf[path]
                             IN_MEMORY = False
                         nMcaInYDataset = 1
@@ -496,11 +509,11 @@ class HDF5Stack1D(DataObject.DataObject):
                                 timePath = NexusTools.getMcaObjectPaths(hdf,
                                                                         path)["elapsed_time"]
                             if timePath in hdf:
-                                timeData = hdf[timePath].value
+                                timeData = hdf[timePath][()]
                             elif "::" in timePath:
                                 externalFile, externalPath = timePath.split("::")
                                 with h5py.File(externalFile, "r") as timeHdf:
-                                    timeData = timeHdf[externalPath].value
+                                    timeData = timeHdf[externalPath][()]
                         if mcaIndex != 0:
                             if IN_MEMORY:
                                 yDataset.shape = -1, mcaDim
@@ -528,8 +541,8 @@ class HDF5Stack1D(DataObject.DataObject):
                                     case = 0
                                     _time[nStart: nStart + nMcaInYDataset] += timeData
                                 if case == -1:
-                                    print("I do not know how to handle this time data")
-                                    print("Ignoring time information")
+                                    _logger.warning("I do not know how to handle this time data")
+                                    _logger.warning("Ignoring time information")
                                     _time= None
                             if (len(yDataset.shape) == 3) and\
                                (dim1 == yDataset.shape[1]):
@@ -641,6 +654,7 @@ class HDF5Stack1D(DataObject.DataObject):
                                         n += 1
                                 else:
                                     n += tmp.shape[1] * tmp.shape[2]
+                        yDataset = None
                         if dim0 == 1:
                             self.onProgress(j)
                 if dim0 != 1:
@@ -661,7 +675,7 @@ class HDF5Stack1D(DataObject.DataObject):
                                 mDataset.shape
                             if xSelection is not None:
                                 xpath = entryName + xSelection
-                                xDataset = hdf[xpath].value
+                                xDataset = hdf[xpath][()]
                         else:
                             path = scan + ySelection
                             if mSelection is not None:
@@ -672,7 +686,7 @@ class HDF5Stack1D(DataObject.DataObject):
                                 mDataset = numpy.asarray(hdf[mpath], dtype=mdtype)
                             if xSelection is not None:
                                 xpath = scan + xSelection
-                                xDataset = hdf[xpath].value
+                                xDataset = hdf[xpath][()]
                         if mSelection is not None:
                             nMonitorData = mDataset.size
                             case = -1
@@ -690,7 +704,7 @@ class HDF5Stack1D(DataObject.DataObject):
                                     "I do not know how to handle this monitor data")
                             if case == 0:
                                 for i in range(yDatasetShape[0]):
-                                    self.data[i] += yDataset[i].value / mDataset[i]
+                                    self.data[i] += yDataset[i][()] / mDataset[i]
                             elif case == 1:
                                 for i in range(yDataset.shape[0]):
                                     self.data[i] += yDataset[i] / mDataset
@@ -703,10 +717,10 @@ class HDF5Stack1D(DataObject.DataObject):
                 nRequiredValues = 1
                 for i in range(len(self.data.shape)):
                     if i != mcaIndex:
-                        nRequiredValues *= self.data.shape[i] 
+                        nRequiredValues *= self.data.shape[i]
                 if _time.size != nRequiredValues:
-                    print("I do not know how to interpret the time information")
-                    print("Ignoring time information")
+                    _logger.warning("I do not know how to interpret the time information")
+                    _logger.warning("Ignoring time information")
                     _time = None
                 else:
                     _time.shape = -1
@@ -732,15 +746,17 @@ class HDF5Stack1D(DataObject.DataObject):
             if xDataset.size == shape[self.info['McaIndex']]:
                 self.x = [xDataset.reshape(-1)]
             else:
-                print("Ignoring xSelection")
+                _logger.warning("Ignoring xSelection")
         elif _channels is not None:
             _channels.shape = -1
             self.x = [_channels]
         if _time is not None:
             self.info["McaLiveTime"] = _time
+        if positionersGroup:
+            self.info["positioners"] = positioners
 
     def getDimensions(self, nFiles, nScans, shape, index=None):
-        #some body may want to overwrite this
+        #somebody may want to overwrite this
         """
         Returns the shape of the final stack as (Dim0, Dim1, Nchannels)
         """
@@ -748,8 +764,7 @@ class HDF5Stack1D(DataObject.DataObject):
             index = -1
         if index == -1:
             index = len(shape) - 1
-        if DEBUG:
-            print("INDEX = %d" % index)
+        _logger.debug("INDEX = %d", index)
         #figure out the shape of the stack
         if len(shape) == 0:
             #a scalar?
@@ -780,9 +795,8 @@ class HDF5Stack1D(DataObject.DataObject):
                 nMca *= shape[i]
 
         mcaDim = shape[index]
-        if DEBUG:
-            print("nMca = %d" % nMca)
-            print("mcaDim = ", mcaDim)
+        _logger.debug("nMca = %d", nMca)
+        _logger.debug("mcaDim = %s", mcaDim)
 
         # HDF allows to work directly from the files without loading
         # them into memory.
