@@ -2,7 +2,7 @@
 #
 # The PyMca X-Ray Fluorescence Toolkit
 #
-# Copyright (c) 2004-2018 European Synchrotron Radiation Facility
+# Copyright (c) 2004-2019 European Synchrotron Radiation Facility
 #
 # This file is part of the PyMca X-ray Fluorescence Toolkit developed at
 # the ESRF by the Software group.
@@ -36,8 +36,10 @@ import sys
 import os
 import struct
 import numpy
+import logging
+
+_logger = logging.getLogger(__name__)
 
-DEBUG = 0
 ALLOW_MULTIPLE_STRIPS = False
 
 TAG_ID  = { 256:"NumberOfColumns",           # S or L ImageWidth
@@ -126,7 +128,7 @@ class TiffIO(object):
             fd = filename
             self._access = None
         else:
-            # the b is needed for windows and python 3
+            # the b is needed to make sure we read bytes
             fd = open(filename, mode)
             self._access = mode
 
@@ -158,8 +160,7 @@ class TiffIO(object):
             if fortyTwo != 42:
                 raise IOError("Invalid TIFF version %d" % fortyTwo)
             else:
-                if DEBUG:
-                    print("VALID TIFF VERSION")
+                _logger.debug("VALID TIFF VERSION")
             if sys.byteorder != fileOrder:
                 swap = True
             else:
@@ -181,8 +182,7 @@ class TiffIO(object):
     def __makeSureFileIsOpen(self):
         if not self.fd.closed:
             return
-        if DEBUG:
-            print("Reopening closed file")
+        _logger.debug("Reopening closed file")
         fileName = self.fd.name
         if self._access is None:
             # we do not own the file
@@ -195,8 +195,7 @@ class TiffIO(object):
     def __makeSureFileIsClosed(self):
         if self._access is None:
             # we do not own the file
-            if DEBUG:
-                print("Not closing not owned file")
+            _logger.debug("Not closing not owned file")
             return
 
         if not self.fd.closed:
@@ -230,25 +229,21 @@ class TiffIO(object):
             offsetToIFD = 0
         else:
             offsetToIFD = struct.unpack(fmt, inStr)[0]
-        if DEBUG:
-            print("Offset to first IFD = %d" % offsetToIFD)
+        _logger.debug("Offset to first IFD = %d", offsetToIFD)
         while offsetToIFD != 0:
             self._IFD.append(offsetToIFD)
             nImages += 1
             fd.seek(offsetToIFD)
             fmt = st + 'H'
             numberOfDirectoryEntries = struct.unpack(fmt,fd.read(struct.calcsize(fmt)))[0]
-            if DEBUG:
-                print("Number of directory entries = %d" % numberOfDirectoryEntries)
+            _logger.debug("Number of directory entries = %d", numberOfDirectoryEntries)
 
             fmt = st + 'I'
             fd.seek(offsetToIFD + 2 + 12 * numberOfDirectoryEntries)
             offsetToIFD = struct.unpack(fmt,fd.read(struct.calcsize(fmt)))[0]
-            if DEBUG:
-                print("Next Offset to IFD = %d" % offsetToIFD)
+            _logger.debug("Next Offset to IFD = %d", offsetToIFD)
             # offsetToIFD = 0
-        if DEBUG:
-            print("Number of images found = %d" % nImages)
+        _logger.debug("Number of images found = %d", nImages)
         return nImages
 
     def _parseImageFileDirectory(self, nImage):
@@ -258,8 +253,7 @@ class TiffIO(object):
         fd.seek(offsetToIFD)
         fmt = st + 'H'
         numberOfDirectoryEntries = struct.unpack(fmt,fd.read(struct.calcsize(fmt)))[0]
-        if DEBUG:
-            print("Number of directory entries = %d" % numberOfDirectoryEntries)
+        _logger.debug("Number of directory entries = %d", numberOfDirectoryEntries)
 
         fmt = st + 'HHI4s'
         tagIDList = []
@@ -292,15 +286,15 @@ class TiffIO(object):
                 valueOffsetList.append(actualValue)
             else:
                 valueOffsetList.append(valueOffset)
-            if DEBUG:
-                if tagID in TAG_ID:
-                    print("tagID = %s" % TAG_ID[tagID])
-                else:
-                    print("tagID        = %d" % tagID)
-                print("fieldType    = %s" % FIELD_TYPE[fieldType][0])
-                print("nValues      = %d" % nValues)
-                # if nValues == 1:
-                #    print("valueOffset =  %s" % valueOffset)
+
+            if tagID in TAG_ID:
+                _logger.debug("tagID = %s", TAG_ID[tagID])
+            else:
+                _logger.debug("tagID        = %d", tagID)
+            _logger.debug("fieldType    = %s", FIELD_TYPE[fieldType][0])
+            _logger.debug("nValues      = %d", nValues)
+            # if nValues == 1:
+            #    print("valueOffset =  %s" % valueOffset)
         return tagIDList, fieldTypeList, nValuesList, valueOffsetList
 
     def _readIFDEntry(self, tag, tagIDList, fieldTypeList, nValuesList, valueOffsetList):
@@ -332,8 +326,7 @@ class TiffIO(object):
                 try:
                     text = raw.decode("utf-8")
                 except UnicodeDecodeError:
-                    if DEBUG:
-                        print("TIFF file tag %d contains non ASCII/UTF-8 characters. " % tag)
+                    _logger.debug("TIFF file tag %d contains non ASCII/UTF-8 characters. ", tag)
                     text = raw.decode("utf-8", errors='replace')
                     # Use a valid ASCII character to limit ferther encoding error
                     text = text.replace(u"\ufffd", "?")
@@ -363,8 +356,7 @@ class TiffIO(object):
 
     def _readInfo(self, nImage, close=True):
         if nImage in self._imageInfoCacheIndex:
-            if DEBUG:
-                print("Reading info from cache")
+            _logger.debug("Reading info from cache")
             return self._imageInfoCache[self._imageInfoCacheIndex.index(nImage)]
 
         # read the header
@@ -424,7 +416,7 @@ class TiffIO(object):
         if TAG_PHOTOMETRIC_INTERPRETATION in tagIDList:
             interpretation = valueOffsetList[tagIDList.index(TAG_PHOTOMETRIC_INTERPRETATION)]
         else:
-            print("WARNING: Non standard TIFF. Photometric interpretation TAG missing")
+            _logger.warning("WARNING: Non standard TIFF. Photometric interpretation TAG missing")
 
         helpString = ""
 
@@ -483,13 +475,13 @@ class TiffIO(object):
                         tagIDList, fieldTypeList, nValuesList, valueOffsetList)[0]
         else:
             rowsPerStrip = nRows
-            print("WARNING: Non standard TIFF. Rows per strip TAG missing")
+            _logger.warning("WARNING: Non standard TIFF. Rows per strip TAG missing")
 
         if TAG_STRIP_BYTE_COUNTS in tagIDList:
             stripByteCounts = self._readIFDEntry(TAG_STRIP_BYTE_COUNTS,
                         tagIDList, fieldTypeList, nValuesList, valueOffsetList)
         else:
-            print("WARNING: Non standard TIFF. Strip byte counts TAG missing")
+            _logger.warning("WARNING: Non standard TIFF. Strip byte counts TAG missing")
             if hasattr(nBits, 'index'):
                 expectedSum = 0
                 for n in nBits:
@@ -509,8 +501,7 @@ class TiffIO(object):
             interpretation = 1
             # we cannot rely on any cache in this case
             useInfoCache = False
-            if DEBUG:
-                print("FORCED MONO")
+            _logger.debug("FORCED MONO")
         else:
             useInfoCache = True
 
@@ -560,8 +551,7 @@ class TiffIO(object):
         return info
 
     def _readImage(self, nImage, **kw):
-        if DEBUG:
-            print("Reading image %d" % nImage)
+        _logger.debug("Reading image %d", nImage)
         if 'close' in kw:
             close = kw['close']
         else:
@@ -569,8 +559,7 @@ class TiffIO(object):
         rowMin = kw.get('rowMin', None)
         rowMax = kw.get('rowMax', None)
         if nImage in self._imageDataCacheIndex:
-            if DEBUG:
-                print("Reading image data from cache")
+            _logger.debug("Reading image data from cache")
             return self._imageDataCache[self._imageDataCacheIndex.index(nImage)]
 
         self.__makeSureFileIsOpen()
@@ -592,8 +581,7 @@ class TiffIO(object):
                 raise IOError("Compressed TIFF images not supported except packbits")
             else:
                 # PackBits compression
-                if DEBUG:
-                    print("Using PackBits compression")
+                _logger.debug("Using PackBits compression")
 
         interpretation = info["photometricInterpretation"]
         if interpretation == 2:
@@ -688,7 +676,7 @@ class TiffIO(object):
             if nRows == rowsPerStrip:
                 actualBytesPerRow = int(image.nbytes / nRows)
                 if actualBytesPerRow != bytesPerRow:
-                    print("Warning: Bogus StripByteCounts information")
+                    _logger.warning("Warning: Bogus StripByteCounts information")
                     bytesPerRow = actualBytesPerRow 
                     nBytes = (rowMax-rowMin+1) * bytesPerRow
             fd.seek(stripOffsets[0] + rowMin * bytesPerRow)
@@ -855,8 +843,7 @@ class TiffIO(object):
 
         # get the image file directories
         nImages = self.getImageFileDirectories()
-        if DEBUG:
-            print("File contains %d images" % nImages)
+        _logger.debug("File contains %d images", nImages)
         if nImages == 0:
             fd.seek(4)
             fmt = st + 'I'
@@ -967,7 +954,7 @@ class TiffIO(object):
                 raw = description
             else:
                 raw = "%s" % description
-                if (sys.version_info > (2,6)) and hasattr(raw, "encode"):
+                if (sys.version_info[:2] > (2,6)) and hasattr(raw, "encode"):
                     raw = raw.encode('utf-8', errors="ignore")
             imageDescription = struct.pack("%ds" % len(raw), raw)
             nDirectoryEntries += 1
@@ -1093,8 +1080,7 @@ class TiffIO(object):
                     stripOffsetsString += struct.pack(fmt, value)
                     stripByteCountsString += struct.pack(fmt, stripByteCounts)
 
-        if DEBUG:
-            print("IMAGE WILL START AT %d" % stripOffsets[0])
+        _logger.debug("IMAGE WILL START AT %d", stripOffsets[0])
 
         # sample format
         if dtype in [numpy.float32, numpy.float64] or\
